@@ -16,9 +16,8 @@ import sys
 # TODO: files that are not yet compliant
 EXCLUDE = (
   "cmd/dot/dot.c",
-  "cmd/dot/dot_builtins.c",
+  "cmd/dot/dot_builtins.cpp",
   "cmd/dot/no_builtins.c",
-  "cmd/dotty/mswin32/dotty.c",
   "cmd/edgepaint/edgepaintmain.c",
   "cmd/gvedit/csettings.cpp",
   "cmd/gvedit/csettings.h",
@@ -39,105 +38,6 @@ EXCLUDE = (
   "cmd/gvmap/power.c",
   "cmd/gvmap/power.h",
   "cmd/gvpr/gvprmain.c",
-  "cmd/lefty/aix_mods/common.h",
-  "cmd/lefty/aix_mods/exec.c",
-  "cmd/lefty/aix_mods/tbl.c",
-  "cmd/lefty/code.c",
-  "cmd/lefty/code.h",
-  "cmd/lefty/common.c",
-  "cmd/lefty/common.h",
-  "cmd/lefty/cs2l/cs2l.c",
-  "cmd/lefty/cs2l/cs2l.h",
-  "cmd/lefty/display.c",
-  "cmd/lefty/display.h",
-  "cmd/lefty/dot2l/dot2l.c",
-  "cmd/lefty/dot2l/dot2l.h",
-  "cmd/lefty/dot2l/dotlex.c",
-  "cmd/lefty/dot2l/dottrie.c",
-  "cmd/lefty/dot2l/triefa.c",
-  "cmd/lefty/dot2l/triefa.h",
-  "cmd/lefty/exec.c",
-  "cmd/lefty/exec.h",
-  "cmd/lefty/g.c",
-  "cmd/lefty/g.h",
-  "cmd/lefty/gfxview.c",
-  "cmd/lefty/gfxview.h",
-  "cmd/lefty/internal.c",
-  "cmd/lefty/internal.h",
-  "cmd/lefty/lefty.c",
-  "cmd/lefty/leftyio.h",
-  "cmd/lefty/lex.c",
-  "cmd/lefty/lex.h",
-  "cmd/lefty/mem.c",
-  "cmd/lefty/mem.h",
-  "cmd/lefty/os/mswin32/io.c",
-  "cmd/lefty/os/unix/io.c",
-  "cmd/lefty/parse.c",
-  "cmd/lefty/parse.h",
-  "cmd/lefty/str.c",
-  "cmd/lefty/str.h",
-  "cmd/lefty/tbl.c",
-  "cmd/lefty/tbl.h",
-  "cmd/lefty/txtview.c",
-  "cmd/lefty/txtview.h",
-  "cmd/lefty/ws/gtk/garray.c",
-  "cmd/lefty/ws/gtk/gbutton.c",
-  "cmd/lefty/ws/gtk/gcanvas.c",
-  "cmd/lefty/ws/gtk/gcommon.c",
-  "cmd/lefty/ws/gtk/gcommon.h",
-  "cmd/lefty/ws/gtk/glabel.c",
-  "cmd/lefty/ws/gtk/gmenu.c",
-  "cmd/lefty/ws/gtk/gpcanvas.c",
-  "cmd/lefty/ws/gtk/gquery.c",
-  "cmd/lefty/ws/gtk/gscroll.c",
-  "cmd/lefty/ws/gtk/gtext.c",
-  "cmd/lefty/ws/gtk/gview.c",
-  "cmd/lefty/ws/mswin32/garray.c",
-  "cmd/lefty/ws/mswin32/gbutton.c",
-  "cmd/lefty/ws/mswin32/gcanvas.c",
-  "cmd/lefty/ws/mswin32/gcommon.c",
-  "cmd/lefty/ws/mswin32/gcommon.h",
-  "cmd/lefty/ws/mswin32/glabel.c",
-  "cmd/lefty/ws/mswin32/gmenu.c",
-  "cmd/lefty/ws/mswin32/gpcanvas.c",
-  "cmd/lefty/ws/mswin32/gquery.c",
-  "cmd/lefty/ws/mswin32/gscroll.c",
-  "cmd/lefty/ws/mswin32/gtext.c",
-  "cmd/lefty/ws/mswin32/gview.c",
-  "cmd/lefty/ws/mswin32/resource.h",
-  "cmd/lefty/ws/none/garray.c",
-  "cmd/lefty/ws/none/gbutton.c",
-  "cmd/lefty/ws/none/gcanvas.c",
-  "cmd/lefty/ws/none/gcommon.c",
-  "cmd/lefty/ws/none/gcommon.h",
-  "cmd/lefty/ws/none/glabel.c",
-  "cmd/lefty/ws/none/gmenu.c",
-  "cmd/lefty/ws/none/gpcanvas.c",
-  "cmd/lefty/ws/none/gquery.c",
-  "cmd/lefty/ws/none/gscroll.c",
-  "cmd/lefty/ws/none/gtext.c",
-  "cmd/lefty/ws/none/gview.c",
-  "cmd/lefty/ws/x11/garray.c",
-  "cmd/lefty/ws/x11/gbutton.c",
-  "cmd/lefty/ws/x11/gcanvas.c",
-  "cmd/lefty/ws/x11/gcommon.c",
-  "cmd/lefty/ws/x11/gcommon.h",
-  "cmd/lefty/ws/x11/glabel.c",
-  "cmd/lefty/ws/x11/gmenu.c",
-  "cmd/lefty/ws/x11/gpcanvas.c",
-  "cmd/lefty/ws/x11/gquery.c",
-  "cmd/lefty/ws/x11/gscroll.c",
-  "cmd/lefty/ws/x11/gtext.c",
-  "cmd/lefty/ws/x11/gview.c",
-  "cmd/lefty/ws/x11/libfilereq/Dir.c",
-  "cmd/lefty/ws/x11/libfilereq/Draw.c",
-  "cmd/lefty/ws/x11/libfilereq/Path.c",
-  "cmd/lefty/ws/x11/libfilereq/SF.h",
-  "cmd/lefty/ws/x11/libfilereq/SFDecls.h",
-  "cmd/lefty/ws/x11/libfilereq/SFinternal.h",
-  "cmd/lefty/ws/x11/libfilereq/SelFile.c",
-  "cmd/lefty/ws/x11/libfilereq/xstat.h",
-  "cmd/lneato/mswin32/lneato.c",
   "cmd/mingle/minglemain.cpp",
   "cmd/smyrna/arcball.c",
   "cmd/smyrna/arcball.h",
@@ -236,7 +136,6 @@ EXCLUDE = (
   "lib/ast/error.h",
   "lib/ast/fmtbuf.c",
   "lib/ast/fmtesc.c",
-  "lib/ast/hashkey.h",
   "lib/ast/pathaccess.c",
   "lib/ast/pathcanon.c",
   "lib/ast/pathcat.c",
@@ -265,7 +164,6 @@ EXCLUDE = (
   "lib/cdt/dtview.c",
   "lib/cdt/dtwalk.c",
   "lib/cgraph/agerror.c",
-  "lib/cgraph/agxbuf.h",
   "lib/cgraph/apply.c",
   "lib/cgraph/attr.c",
   "lib/cgraph/cghdr.h",
@@ -328,7 +226,6 @@ EXCLUDE = (
   "lib/common/intset.c",
   "lib/common/intset.h",
   "lib/common/labels.c",
-  "lib/common/logic.h",
   "lib/common/macros.h",
   "lib/common/memory.c",
   "lib/common/memory.h",
