@@ -13,8 +13,11 @@
 #include <sparse/general.h>
 #include <math.h>
 #include <sparse/QuadTree.h>
+#include <stdbool.h>
 #include <string.h>
 #include <cgraph/agxbuf.h>
+#include <cgraph/alloc.h>
+#include <cgraph/prisize_t.h>
 #include <cgraph/cgraph.h>
 #include "make_map.h"
 #include <sfdpgen/stress_model.h>
@@ -22,16 +25,10 @@
 #include <sparse/colorutil.h>
 #include <neatogen/delaunay.h>
 
-#ifdef SINGLE
-#define REAL float
-#else /* not SINGLE */
-#define REAL double
-#endif /* not SINGLE */
-
 #include <edgepaint/lab.h>
 #include <edgepaint/node_distinct_coloring.h>
 
-void map_palette_optimal_coloring(char *color_scheme, char *lightness, SparseMatrix A0, real accuracy, int seed, 
+void map_palette_optimal_coloring(char *color_scheme, char *lightness, SparseMatrix A0, double accuracy, int seed, 
 				  float **rgb_r, float **rgb_g, float **rgb_b){
   /*
     for a graph A, get a distinctive color of its nodes so that the color distanmce among all nodes are maximized. Here
@@ -50,39 +47,38 @@ void map_palette_optimal_coloring(char *color_scheme, char *lightness, SparseMat
  
   /*color: On input an array of size n*cdim, if NULL, will be allocated. On exit the final color assignment for node i is [cdim*i,cdim*(i+1)), in RGB (between 0 to 1)
   */
-  real *colors = NULL;
+  double *colors = NULL;
   int n = A0->m, i, cdim;
 
   SparseMatrix A;
-  int weightedQ = TRUE;
-  int iter_max = 100;
+  bool weightedQ = true;
 
-  {real *dist = NULL;
-    A = SparseMatrix_symmetrize(A0, FALSE);
+  {double *dist = NULL;
+    A = SparseMatrix_symmetrize(A0, false);
     SparseMatrix_distance_matrix(A, 0, &dist);
     SparseMatrix_delete(A);
     A = SparseMatrix_from_dense(n, n, dist);
-    FREE(dist);
+    free(dist);
     A = SparseMatrix_remove_diagonal(A);
     SparseMatrix_export(stdout, A);
   }
 
-  node_distinct_coloring(color_scheme, lightness, weightedQ, A, accuracy, iter_max, seed, &cdim, &colors);
+  node_distinct_coloring(color_scheme, lightness, weightedQ, A, accuracy, seed,
+                         &cdim, &colors);
 
   if (A != A0){
     SparseMatrix_delete(A);
   }
-  *rgb_r = MALLOC(sizeof(float)*(n+1));
-  *rgb_g = MALLOC(sizeof(float)*(n+1));
-  *rgb_b = MALLOC(sizeof(float)*(n+1));
+  *rgb_r = gv_calloc(n + 1, sizeof(float));
+  *rgb_g = gv_calloc(n + 1, sizeof(float));
+  *rgb_b = gv_calloc(n + 1, sizeof(float));
 
   for (i = 0; i < n; i++){
     (*rgb_r)[i+1] = (float) colors[cdim*i];
     (*rgb_g)[i+1] = (float) colors[cdim*i + 1];
     (*rgb_b)[i+1] = (float) colors[cdim*i + 2];
   }
-  FREE(colors);
-
+  free(colors);
 }
 
 void map_optimal_coloring(int seed, SparseMatrix A, float *rgb_r,  float *rgb_g, float *rgb_b){
@@ -100,33 +96,32 @@ void map_optimal_coloring(int seed, SparseMatrix A, float *rgb_r,  float *rgb_g,
   for (i = 0; i < n; i++) rgb_g[i] = u[i];
   vector_float_take(n, rgb_b, n, p, &u);
   for (i = 0; i < n; i++) rgb_b[i] = u[i];
-  FREE(u);
-
+  free(u);
 }
 
 static int get_poly_id(int ip, SparseMatrix point_poly_map){
   return point_poly_map->ja[point_poly_map->ia[ip]];
 }
  
-void improve_contiguity(int n, int dim, int *grouping, SparseMatrix poly_point_map, real *x, SparseMatrix graph){
+void improve_contiguity(int n, int dim, int *grouping, SparseMatrix poly_point_map, double *x, SparseMatrix graph){
  /* 
      grouping: which group each of the vertex belongs to
      poly_point_map: a matrix of dimension npolys x (n + nrandom), poly_point_map[i,j] != 0 if polygon i contains the point j.
      .  If j < n, it is the original point, otherwise it is artificial point (forming the rectangle around a label) or random points.
   */
   int i, j, *ia, *ja, u, v;
-  real *a;
+  double *a;
   SparseMatrix point_poly_map, D;
-  real dist;
+  double dist;
   int nbad = 0, flag;
   int maxit = 10;
-  real tol = 0.001;
+  double tol = 0.001;
 
   D = SparseMatrix_get_real_adjacency_matrix_symmetrized(graph);
 
   assert(graph->m == n);
   ia = D->ia; ja = D->ja;
-  a = (real*) D->a;
+  a = (double*) D->a;
 
   /* point_poly_map: each row i has only 1 entry at column j, which says that point i is in polygon j */
   point_poly_map = SparseMatrix_transpose(poly_point_map);
@@ -148,8 +143,8 @@ void improve_contiguity(int n, int dim, int *grouping, SparseMatrix poly_point_m
     }
   }
 
-  if (Verbose) fprintf(stderr,"ratio (edges among discontiguous regions vs total edges)=%f\n",((real) nbad)/ia[n]);
-  stress_model(dim, D, D, &x, FALSE, maxit, tol, &flag);
+  if (Verbose) fprintf(stderr,"ratio (edges among discontiguous regions vs total edges)=%f\n",((double) nbad)/ia[n]);
+  stress_model(dim, D, &x, FALSE, maxit, tol, &flag);
 
   assert(!flag);
 
@@ -159,10 +154,10 @@ void improve_contiguity(int n, int dim, int *grouping, SparseMatrix poly_point_m
 
 struct Triangle {
   int vertices[3];/* 3 points */
-  real center[2]; /* center of the triangle */
+  double center[2]; /* center of the triangle */
 };
 
-static void normal(real v[], real normal[]){
+static void normal(double v[], double normal[]){
   if (v[0] == 0){
     normal[0] = 1; normal[1] = 0;
   } else {
@@ -171,7 +166,7 @@ static void normal(real v[], real normal[]){
   }
 }
 
-static void triangle_center(real x[], real y[], real z[], real c[]){
+static void triangle_center(double x[], double y[], double z[], double c[]){
   /* find the "center" c, which is the intersection of the 3 vectors that are normal to each
      of the edges respectively, and which passes through the center of the edges respectively
      center[{x_, y_, z_}] := Module[
@@ -186,7 +181,7 @@ static void triangle_center(real x[], real y[], real z[], real c[]){
      
      ]
  */
-  real xy[2], yz[2], nxy[2], nyz[2], ymx[2], ymz[2], beta, bot;
+  double xy[2], yz[2], nxy[2], nyz[2], ymx[2], ymz[2], beta, bot;
   int i;
 
   for (i = 0; i < 2; i++) ymx[i] = y[i] - x[i];
@@ -212,8 +207,8 @@ static SparseMatrix matrix_add_entry(SparseMatrix A, int i, int j, int val){
   if (i < j) {
     i1 = j; j1 = i;
   }
-  A = SparseMatrix_coordinate_form_add_entries(A, 1, &j1, &i1, &val);
-  return SparseMatrix_coordinate_form_add_entries(A, 1, &i1, &j1, &val);
+  A = SparseMatrix_coordinate_form_add_entry(A, j1, i1, &val);
+  return SparseMatrix_coordinate_form_add_entry(A, i1, j1, &val);
 }
 
 static void plot_dot_edges(FILE *f, SparseMatrix A){
@@ -231,7 +226,7 @@ static void plot_dot_edges(FILE *f, SparseMatrix A){
   }
 }
 
-static void plot_dot_labels(FILE *f, int n, int dim, real *x, char **labels, float *fsz){
+static void plot_dot_labels(FILE *f, int n, int dim, double *x, char **labels, float *fsz){
   int i;
 
   for (i = 0; i < n; i++){
@@ -244,19 +239,21 @@ static void plot_dot_labels(FILE *f, int n, int dim, real *x, char **labels, flo
 
 }
 
-static void dot_polygon(agxbuf *sbuff, int np, float *xp, float *yp, real line_width,  
+static void dot_polygon(agxbuf *sbuff, int np, float *xp, float *yp, double line_width,  
                         int fill, const char *cstring){
 
   if (np > 0){
     if (fill >= 0){
-      agxbprint(sbuff, " c %zu -%s C %zu -%s P %d ", strlen(cstring), cstring, strlen(cstring), cstring, np);
+      agxbprint(sbuff, " c %" PRISIZE_T " -%s C %" PRISIZE_T " -%s P %d ",
+                strlen(cstring), cstring, strlen(cstring), cstring, np);
     } else {
       if (line_width > 0){
 	size_t len_swidth = (size_t)snprintf(NULL, 0, "%f", line_width);
-	agxbprint(sbuff, " c %zu -%s S %zu -setlinewidth(%f) L %d ",
-	          strlen(cstring), cstring, len_swidth + 14, line_width, np);
+	agxbprint(sbuff, " c %" PRISIZE_T " -%s S %" PRISIZE_T
+	          " -setlinewidth(%f) L %d ", strlen(cstring), cstring,
+	          len_swidth + 14, line_width, np);
       } else {
-	agxbprint(sbuff, " c %zu -%s L %d ", strlen(cstring), cstring, np);
+	agxbprint(sbuff, " c %" PRISIZE_T " -%s L %d ", strlen(cstring), cstring, np);
       }
     }
     for (int i = 0; i < np; i++){
@@ -265,14 +262,14 @@ static void dot_polygon(agxbuf *sbuff, int np, float *xp, float *yp, real line_w
   }
 }
 
-static void dot_one_poly(agxbuf *sbuff, real line_width, int fill, int np,
+static void dot_one_poly(agxbuf *sbuff, double line_width, int fill, int np,
                          float *xp, float *yp, const char *cstring) {
   dot_polygon(sbuff, np, xp, yp, line_width, fill, cstring);
 }
 
-static void plot_dot_polygons(agxbuf *sbuff, real line_width,
+static void plot_dot_polygons(agxbuf *sbuff, double line_width,
                               const char *line_color, SparseMatrix polys,
-                              real *x_poly, int *polys_groups, float *r,
+                              double *x_poly, int *polys_groups, float *r,
                               float *g, float *b, const char *opacity) {
   int i, j, *ia = polys->ia, *ja = polys->ja, *a = (int*) polys->a, npolys = polys->m, nverts = polys->n, ipoly,first;
   int np = 0, maxlen = 0;
@@ -283,8 +280,8 @@ static void plot_dot_polygons(agxbuf *sbuff, real line_width,
   
   for (i = 0; i < npolys; i++) maxlen = MAX(maxlen, ia[i+1]-ia[i]);
 
-  xp = MALLOC(sizeof(float)*maxlen);
-  yp = MALLOC(sizeof(float)*maxlen);
+  xp = gv_calloc(maxlen, sizeof(float));
+  yp = gv_calloc(maxlen, sizeof(float));
 
   if (Verbose) fprintf(stderr,"npolys = %d\n",npolys);
   first = abs(a[0]); ipoly = first + 1;
@@ -309,23 +306,20 @@ static void plot_dot_polygons(agxbuf *sbuff, real line_width,
       dot_one_poly(sbuff, -1, 1, np, xp, yp, cstring);
     }
   }
-  FREE(xp);
-  FREE(yp);
-
+  free(xp);
+  free(yp);
 }
 
-void plot_dot_map(Agraph_t* gr, int n, int dim, real *x, SparseMatrix polys,
-                  SparseMatrix poly_lines, real line_width,
-                  const char *line_color, real *x_poly, int *polys_groups,
+void plot_dot_map(Agraph_t* gr, int n, int dim, double *x, SparseMatrix polys,
+                  SparseMatrix poly_lines, double line_width,
+                  const char *line_color, double *x_poly, int *polys_groups,
                   char **labels, float *fsz, float *r, float *g, float *b,
                   const char* opacity, SparseMatrix A, FILE* f) {
   /* if graph object exist, we just modify some attributes, otherwise we dump the whole graph */
-  int plot_polyQ = TRUE;
-  agxbuf sbuff;
+  bool plot_polyQ = true;
+  agxbuf sbuff = {0};
 
-  agxbinit(&sbuff, 0, NULL);
-
-  if (!r || !g || !b) plot_polyQ = FALSE;
+  if (!r || !g || !b) plot_polyQ = false;
 
   if (!gr) {
     fprintf(f, "graph map {\n node [margin = 0 width=0.0001 height=0.00001 shape=plaintext];\n graph [outputorder=edgesfirst, bgcolor=\"#dae2ff\"]\n edge [color=\"#55555515\",fontname=\"Helvetica-Bold\"]\n");
@@ -371,7 +365,7 @@ void plot_dot_map(Agraph_t* gr, int n, int dim, real *x, SparseMatrix polys,
   agxbfree(&sbuff);
 }
 
-static void get_tri(int n, int dim, real *x, int *nt, struct Triangle **T, SparseMatrix *E) {
+static void get_tri(int n, int dim, double *x, int *nt, struct Triangle **T, SparseMatrix *E) {
    /* always contains a self edge and is symmetric.
      input:
      n: number of points
@@ -388,7 +382,7 @@ static void get_tri(int n, int dim, real *x, int *nt, struct Triangle **T, Spars
 
   int* trilist = get_triangles(x, n, &ntri);
 
-  *T = N_NEW(ntri,struct Triangle);
+  *T = gv_calloc(ntri, sizeof(struct Triangle));
 
   A = SparseMatrix_new(n, n, 1, MATRIX_TYPE_INTEGER, FORMAT_COORD);
   for (i = 0; i < ntri; i++) {
@@ -410,7 +404,7 @@ static void get_tri(int n, int dim, real *x, int *nt, struct Triangle **T, Spars
 
   *nt = ntri;
 
-  FREE(trilist);
+  free(trilist);
 }
 
 static SparseMatrix get_country_graph(int n, SparseMatrix A, int *groups, int GRP_RANDOM, int GRP_BBOX){
@@ -432,12 +426,12 @@ static SparseMatrix get_country_graph(int n, SparseMatrix A, int *groups, int GR
   ja = A->ja;
   for (i = 0; i < n; i++){
     ig1 = groups[i]-1;/* add a diagonal entry */
-    SparseMatrix_coordinate_form_add_entries(B, 1, &ig1, &ig1, &one);
+    SparseMatrix_coordinate_form_add_entry(B, ig1, ig1, &one);
     for (j = ia[i]; j < ia[i+1]; j++){
       jj = ja[j];
       if (i != jj && groups[i] != groups[jj] && groups[jj] != GRP_RANDOM && groups[jj] != GRP_BBOX){
 	ig1 = groups[i]-1; ig2 = groups[jj]-1;
-	SparseMatrix_coordinate_form_add_entries(B, 1, &ig1, &ig2, &one);
+	SparseMatrix_coordinate_form_add_entry(B, ig1, ig2, &one);
       }
     }
   }
@@ -460,7 +454,7 @@ static void conn_comp(int n, SparseMatrix A, int *groups, SparseMatrix *poly_poi
     for (j = ia[i]; j < ia[i+1]; j++){
       jj = ja[j];
       if (i != jj && groups[i] == groups[jj]){
-	SparseMatrix_coordinate_form_add_entries(B, 1, &i, &jj, &one);
+	SparseMatrix_coordinate_form_add_entry(B, i, jj, &one);
       }
     }
   }
@@ -470,8 +464,8 @@ static void conn_comp(int n, SparseMatrix A, int *groups, SparseMatrix *poly_poi
   SparseMatrix_delete(B);
   SparseMatrix_delete(BB);
   *poly_point_map = SparseMatrix_new(ncomps, n, n, MATRIX_TYPE_PATTERN, FORMAT_CSR);
-  FREE((*poly_point_map)->ia);
-  FREE((*poly_point_map)->ja);
+  free((*poly_point_map)->ia);
+  free((*poly_point_map)->ja);
   (*poly_point_map)->ia = comps_ptr;
   (*poly_point_map)->ja = comps;
   (*poly_point_map)->nz = n;
@@ -500,7 +494,7 @@ static void get_poly_lines(int exclude_random, int nt, SparseMatrix graph, Spars
   graph = NULL;/* we disable checking whether a polyline cross an edge for now due to issues with labels */
   if (graph) {
     assert(graph->m == n);
-    gmask = malloc(sizeof(int)*n);
+    gmask = gv_calloc(n, sizeof(int));
     for (i = 0; i < n; i++) gmask[i] = -1;
     ia = graph->ia; ja = graph->ja;
     edim = 5;/* we also store info about whether an edge of a polygon corresponds to a real edge or not. */
@@ -508,10 +502,10 @@ static void get_poly_lines(int exclude_random, int nt, SparseMatrix graph, Spars
 
   for (i = 0; i < nt; i++) mask[i] = -1;
   /* loop over every point in each connected component */
-  elist = MALLOC(sizeof(int)*nt*edim);
-  tlist = MALLOC(sizeof(int)*nt*2);
+  elist = gv_calloc(nt * edim, sizeof(int));
+  tlist = gv_calloc(nt * 2, sizeof(int));
   *poly_lines = SparseMatrix_new(ncomps, nt, 1, MATRIX_TYPE_INTEGER, FORMAT_COORD);
-  *polys_groups = MALLOC(sizeof(int)*ncomps);
+  *polys_groups = gv_calloc(ncomps, sizeof(int));
 
   for (i = 0; i < nt; i++) elist[i*edim + 2] = 0;
   nz = ie[E->m] - ie[0];
@@ -595,7 +589,7 @@ static void get_poly_lines(int exclude_random, int nt, SparseMatrix graph, Spars
 	} else {
 	  ipoly2 = ipoly;
 	}
-	SparseMatrix_coordinate_form_add_entries(*poly_lines, 1, &i, &cur, &ipoly2);
+	SparseMatrix_coordinate_form_add_entry(*poly_lines, i, cur, &ipoly2);
 	while (next != sta){
 	  mask[next] = i;
 	  
@@ -604,7 +598,7 @@ static void get_poly_lines(int exclude_random, int nt, SparseMatrix graph, Spars
 	  } else {
 	    ipoly2 = ipoly;
 	  }
-	  SparseMatrix_coordinate_form_add_entries(*poly_lines, 1, &i, &next, &ipoly2);
+	  SparseMatrix_coordinate_form_add_entry(*poly_lines, i, next, &ipoly2);
 
 	  nn = neighbor(next, 0, edim, elist);
 	  nlink = 0;
@@ -623,7 +617,7 @@ static void get_poly_lines(int exclude_random, int nt, SparseMatrix graph, Spars
 	} else {
 	  ipoly2 = ipoly;
 	}
-	SparseMatrix_coordinate_form_add_entries(*poly_lines, 1, &i, &sta, &ipoly2);/* complete a cycle by adding starting point */
+	SparseMatrix_coordinate_form_add_entry(*poly_lines, i, sta, &ipoly2);/* complete a cycle by adding starting point */
 
 	ipoly++;
       }
@@ -635,13 +629,13 @@ static void get_poly_lines(int exclude_random, int nt, SparseMatrix graph, Spars
   SparseMatrix_delete(*poly_lines);
   *poly_lines = A;
 
-  FREE(tlist);
-  FREE(elist);
+  free(tlist);
+  free(elist);
 }
 
-static void plot_cycle(int head, int *cycle, int *edge_table, real *x){
+static void plot_cycle(int head, int *cycle, int *edge_table, double *x){
   int cur, next;
-  real x1, x2, y1, y2;
+  double x1, x2, y1, y2;
 
   printf("Graphics[{");
   cur = head;
@@ -688,7 +682,7 @@ static int same_edge(int ecur, int elast, int *edge_table){
 }
 
 static void get_polygon_solids(int nt, SparseMatrix E, int ncomps, int *comps_ptr, int *comps,
-			       int *mask, real *x_poly, SparseMatrix *polys){
+			       int *mask, double *x_poly, SparseMatrix *polys){
   /*============================================================
 
     polygon slids that will be colored
@@ -720,7 +714,7 @@ static void get_polygon_solids(int nt, SparseMatrix E, int ncomps, int *comps_pt
   SparseMatrix B;
 
   ne = E->nz;
-  edge_table = MALLOC(sizeof(int)*ne*2);  
+  edge_table = gv_calloc(ne * 2, sizeof(int));
 
   for (i = 0; i < n; i++) mask[i] = -1;
 
@@ -736,14 +730,14 @@ static void get_polygon_solids(int nt, SparseMatrix E, int ncomps, int *comps_pt
 	assert(jj < n);
 	edge_table[ne*2] = t1;/*t1->t2*/
 	edge_table[ne*2+1] = t2;
-	half_edges = SparseMatrix_coordinate_form_add_entries(half_edges, 1, &i, &jj, &ne);
-	half_edges = SparseMatrix_coordinate_form_add_entries(half_edges, 1, &jj, &i, &ne);
+	half_edges = SparseMatrix_coordinate_form_add_entry(half_edges, i, jj, &ne);
+	half_edges = SparseMatrix_coordinate_form_add_entry(half_edges, jj, i, &ne);
 	ne++;
 
 	edge_table[ne*2] = t2;/*t2->t1*/
 	edge_table[ne*2+1] = t1;
-	half_edges = SparseMatrix_coordinate_form_add_entries(half_edges, 1, &i, &jj, &ne);
-	half_edges = SparseMatrix_coordinate_form_add_entries(half_edges, 1, &jj, &i, &ne);	
+	half_edges = SparseMatrix_coordinate_form_add_entry(half_edges, i, jj, &ne);
+	half_edges = SparseMatrix_coordinate_form_add_entry(half_edges, jj, i, &ne);	
 
 
 	ne++;
@@ -753,19 +747,19 @@ static void get_polygon_solids(int nt, SparseMatrix E, int ncomps, int *comps_pt
   }
   assert(E->nz >= ne);
 
-  cycle = MALLOC(sizeof(int)*ne*2);
+  cycle = gv_calloc(ne * 2, sizeof(int));
   B = SparseMatrix_from_coordinate_format_not_compacted(half_edges);
   SparseMatrix_delete(half_edges);half_edges = B;
 
-  edge_cycle_map = MALLOC(sizeof(int)*ne);
-  emask = MALLOC(sizeof(int)*ne);
+  edge_cycle_map = gv_calloc(ne, sizeof(int));
+  emask = gv_calloc(ne, sizeof(int));
   for (i = 0; i < ne; i++) edge_cycle_map[i] = NOT_ON_CYCLE;
   for (i = 0; i < ne; i++) emask[i] = -1;
 
   ie = half_edges->ia;
   je = half_edges->ja;
   e = (int*) half_edges->a;
-  elist = MALLOC(sizeof(int)*nt*3);
+  elist = gv_calloc(nt * 3, sizeof(int));
   for (i = 0; i < nt; i++) elist[i*edim + 2] = 0;
 
   *polys = SparseMatrix_new(ncomps, nt, 1, MATRIX_TYPE_INTEGER, FORMAT_COORD);
@@ -922,13 +916,13 @@ static void get_polygon_solids(int nt, SparseMatrix E, int ncomps, int *comps_pt
     while ((enext = cycle_next(ecur)) != cycle_head){
       edge_cycle_map[ecur] = NOT_ON_CYCLE;
       head = edge_head(ecur);
-      SparseMatrix_coordinate_form_add_entries(*polys, 1, &i, &head, &i);
+      SparseMatrix_coordinate_form_add_entry(*polys, i, head, &i);
       ecur = enext;
     }
     edge_cycle_map[ecur] = NOT_ON_CYCLE;
     head = edge_head(ecur); tail = edge_tail(ecur);
-    SparseMatrix_coordinate_form_add_entries(*polys, 1, &i, &head, &i);
-    SparseMatrix_coordinate_form_add_entries(*polys, 1, &i, &tail, &i);
+    SparseMatrix_coordinate_form_add_entry(*polys, i, head, &i);
+    SparseMatrix_coordinate_form_add_entry(*polys, i, tail, &i);
 
 
     /* unset edge_map */
@@ -939,15 +933,15 @@ static void get_polygon_solids(int nt, SparseMatrix E, int ncomps, int *comps_pt
   *polys = B;
   
   SparseMatrix_delete(half_edges);
-  FREE(cycle);
-  FREE(edge_cycle_map);
-  FREE(elist);
-  FREE(emask);
-  FREE(edge_table);
-
+  free(cycle);
+  free(edge_cycle_map);
+  free(elist);
+  free(emask);
+  free(edge_table);
 }
+
 static void get_polygons(int exclude_random, int n, int nrandom, int dim, SparseMatrix graph, int *grouping,
-			 int nt, struct Triangle *Tp, SparseMatrix E, int *nverts, real **x_poly, 
+			 int nt, struct Triangle *Tp, SparseMatrix E, int *nverts, double **x_poly, 
 			 int *npolys, SparseMatrix *poly_lines, SparseMatrix *polys, int **polys_groups, SparseMatrix *poly_point_map, SparseMatrix *country_graph){
   int i, j;
   int *mask;
@@ -960,7 +954,7 @@ static void get_polygons(int exclude_random, int n, int nrandom, int dim, Sparse
   assert(dim == 2);
   *nverts = nt;
  
-  groups = MALLOC(sizeof(int)*(n + nrandom));
+  groups = gv_calloc(n + nrandom, sizeof(int));
   maxgrp = grouping[0];
   for (i = 0; i < n; i++) {
     maxgrp = MAX(maxgrp, grouping[i]);
@@ -976,7 +970,7 @@ static void get_polygons(int exclude_random, int n, int nrandom, int dim, Sparse
   }
   
   /* finding connected components: vertices that are connected in the triangle graph, as well as in the same group */
-  mask = MALLOC(sizeof(int)*MAX(n + nrandom, 2*nt));
+  mask = gv_calloc(MAX(n + nrandom, 2 * nt), sizeof(int));
   conn_comp(n + nrandom, E, groups, poly_point_map);
 
   ncomps = (*poly_point_map)->m;
@@ -1001,7 +995,7 @@ static void get_polygons(int exclude_random, int n, int nrandom, int dim, Sparse
   }
   *npolys = ncomps;
 
-  *x_poly = MALLOC(sizeof(real)*dim*nt);
+  *x_poly = gv_calloc(dim * nt, sizeof(double));
   for (i = 0; i < nt; i++){
     for (j = 0; j < dim; j++){
       (*x_poly)[i*dim+j] = Tp[i].center[j];
@@ -1025,35 +1019,34 @@ static void get_polygons(int exclude_random, int n, int nrandom, int dim, Sparse
   B = get_country_graph(n, E, groups, GRP_RANDOM, GRP_BBOX);
   *country_graph = B;
 
-  FREE(groups);
-  FREE(mask);
+  free(groups);
+  free(mask);
 }
 
 static int make_map_internal(int exclude_random, int include_OK_points,
-		      int n, int dim, real *x0, int *grouping0, SparseMatrix graph, real bounding_box_margin[], int *nrandom, int nedgep, 
-		      real shore_depth_tol, real **xcombined, int *nverts, real **x_poly, 
+		      int n, int dim, double *x0, int *grouping0, SparseMatrix graph, double bounding_box_margin[], int *nrandom, int nedgep, 
+		      double shore_depth_tol, double **xcombined, int *nverts, double **x_poly, 
 		      int *npolys, SparseMatrix *poly_lines, SparseMatrix *polys, int **polys_groups, SparseMatrix *poly_point_map,
-		      SparseMatrix *country_graph, int highlight_cluster, int *flag){
+		      SparseMatrix *country_graph, int highlight_cluster){
 
 
-  real xmax[2], xmin[2], area, *x = x0;
+  double xmax[2], xmin[2], area, *x = x0;
   int i, j;
   QuadTree qt;
   int dim2 = 2, nn = 0;
   int max_qtree_level = 10;
-  real ymin[2], min;
+  double ymin[2], min;
   int imin, nz, nzok = 0, nzok0 = 0, nt;
-  real *xran, point[2];
+  double *xran, point[2];
   struct Triangle *Tp;
   SparseMatrix E;
-  real boxsize[2];
+  double boxsize[2];
   int INCLUDE_OK_POINTS = include_OK_points;/* OK points are random points inserted and found to be within shore_depth_tol of real/artificial points,
 			      including them instead of throwing away increase realism of boundary */
   int *grouping = grouping0;
 
   int HIGHLIGHT_SET = highlight_cluster;
 
-  *flag = 0;
   for (j = 0; j < dim2; j++) {
     xmax[j] = x[j];
     xmin[j] = x[j];
@@ -1079,7 +1072,7 @@ static int make_map_internal(int exclude_random, int include_OK_points,
     *nrandom -= 4;
   }
 
-  if (shore_depth_tol < 0) shore_depth_tol = sqrt(area/(real) n); /* set to average distance for random distribution */
+  if (shore_depth_tol < 0) shore_depth_tol = sqrt(area/(double) n); /* set to average distance for random distribution */
   if (Verbose) fprintf(stderr,"nrandom=%d shore_depth_tol=%.08f\n",*nrandom, shore_depth_tol);
 
 
@@ -1087,14 +1080,14 @@ static int make_map_internal(int exclude_random, int include_OK_points,
      two connected components be separated due to small shore depth */
   {
     int nz;
-    real *y;
+    double *y;
     int k, t, np=nedgep;
     if (graph && np){
       fprintf(stderr,"add art np = %d\n",np);
       nz = graph->nz;
-      y = MALLOC(sizeof(real)*(dim*n + dim*nz*np));
+      y = gv_calloc(dim * n + dim * nz * np, sizeof(double));
       for (i = 0; i < n*dim; i++) y[i] = x[i];
-      grouping = MALLOC(sizeof(int)*(n + nz*np));
+      grouping = gv_calloc(n + nz * np, sizeof(int));
       for (i = 0; i < n; i++) grouping[i] = grouping0[i];
       nz = n;
       for (i = 0; i < graph->m; i++){
@@ -1103,10 +1096,10 @@ static int make_map_internal(int exclude_random, int include_OK_points,
 	  if (!HIGHLIGHT_SET || (grouping[i] == grouping[graph->ja[j]] && grouping[i] == HIGHLIGHT_SET)){
 	    for (t = 0; t < np; t++){
 	      for (k = 0; k < dim; k++){
-		y[nz*dim+k] = t/((real) np)*x[i*dim+k] + (1-t/((real) np))*x[(graph->ja[j])*dim + k];
+		y[nz*dim+k] = t/((double) np)*x[i*dim+k] + (1-t/((double) np))*x[(graph->ja[j])*dim + k];
 	      }
 	      assert(n + (nz-n)*np + t < n + nz*np && n + (nz-n)*np + t >= 0);
-	      if (t/((real) np) > 0.5){
+	      if (t/((double) np) > 0.5){
 		grouping[nz] = grouping[i];
 	      } else {
 		grouping[nz] = grouping[graph->ja[j]];
@@ -1119,9 +1112,9 @@ static int make_map_internal(int exclude_random, int include_OK_points,
       fprintf(stderr, "after adding edge points, n:%d->%d\n",n, nz);
       n = nz;
       x = y;
-      qt = QuadTree_new_from_point_list(dim, nz, max_qtree_level, y, NULL);
+      qt = QuadTree_new_from_point_list(dim, nz, max_qtree_level, y);
     } else {
-      qt = QuadTree_new_from_point_list(dim, n, max_qtree_level, x, NULL);
+      qt = QuadTree_new_from_point_list(dim, n, max_qtree_level, x);
     }
   }
   graph = NULL;
@@ -1142,8 +1135,8 @@ static int make_map_internal(int exclude_random, int include_OK_points,
       }
     }
     if (Verbose) {
-      real bbm0 = bounding_box_margin[0];
-      real bbm1 = bounding_box_margin[1];
+      double bbm0 = bounding_box_margin[0];
+      double bbm1 = bounding_box_margin[1];
       if (bbm0 > 0)
 	fprintf (stderr, "bounding box margin: %.06f", bbm0);
       else if (bbm0 < 0)
@@ -1158,23 +1151,23 @@ static int make_map_internal(int exclude_random, int include_OK_points,
 	fprintf (stderr, " %.06f\n", MAX(boxsize[1]*0.2, 2*shore_depth_tol));
     }
     if (*nrandom < 0) {
-      real n1, n2, area2;
+      double n1, n2, area2;
       area2 = (xmax[1] - xmin[1])*(xmax[0] - xmin[0]);
       n1 = (int) area2/(shore_depth_tol*shore_depth_tol);
       n2 = n*((int) area2/area);
       *nrandom = MAX(n1, n2);
     }
     srand(123);
-    xran = MALLOC(sizeof(real)*(*nrandom + 4)*dim2);
+    xran = gv_calloc((*nrandom + 4) * dim2, sizeof(double));
     nz = 0;
     if (INCLUDE_OK_POINTS){
       nzok0 = nzok = *nrandom - 1;/* points that are within tolerance of real or artificial points */
       if (grouping == grouping0) {
-        int *grouping2 = MALLOC(sizeof(int)*(n + *nrandom));
+        int *grouping2 = gv_calloc(n + *nrandom, sizeof(int));
         memcpy(grouping2, grouping, sizeof(int)*n);
         grouping = grouping2;
       } else {
-        grouping = REALLOC(grouping, sizeof(int)*(n + *nrandom));
+        grouping = gv_recalloc(grouping, n, n + *nrandom, sizeof(int));
       }
     }
     nn = n;
@@ -1185,8 +1178,7 @@ static int make_map_internal(int exclude_random, int include_OK_points,
 	point[j] = xmin[j] + (xmax[j] - xmin[j])*drand();
       }
       
-      QuadTree_get_nearest(qt, point, ymin, &imin, &min, flag);
-      assert(!(*flag));
+      QuadTree_get_nearest(qt, point, ymin, &imin, &min);
 
       if (min > shore_depth_tol){/* point not too close, accepted */
 	for (j = 0; j < dim2; j++){
@@ -1206,7 +1198,7 @@ static int make_map_internal(int exclude_random, int include_OK_points,
     *nrandom = nz;
     if (Verbose) fprintf( stderr, "nn nrandom=%d\n",*nrandom);
   } else {
-    xran = MALLOC(sizeof(real)*4*dim2);
+    xran = gv_calloc(4 * dim2, sizeof(double));
   }
 
 
@@ -1226,9 +1218,9 @@ static int make_map_internal(int exclude_random, int include_OK_points,
 
 
   if (INCLUDE_OK_POINTS){
-    *xcombined = MALLOC(sizeof(real)*(nn+*nrandom)*dim2);
+    *xcombined = gv_calloc((nn + *nrandom) * dim2, sizeof(double));
   } else {
-    *xcombined = MALLOC(sizeof(real)*(n+*nrandom)*dim2);
+    *xcombined = gv_calloc((n + *nrandom) * dim2, sizeof(double));
   }
   for (i = 0; i < n; i++) {
     for (j = 0; j < dim2; j++) (*xcombined)[i*dim2+j] = x[i*dim+j];
@@ -1247,10 +1239,10 @@ static int make_map_internal(int exclude_random, int include_OK_points,
 
   {
     int nz, nh = 0;/* the set to highlight */
-    real *xtemp;
+    double *xtemp;
     if (HIGHLIGHT_SET){
       if (Verbose) fprintf(stderr," highlight cluster %d, n = %d\n",HIGHLIGHT_SET, n);
-      xtemp = MALLOC(sizeof(real)*n*dim);
+      xtemp = gv_calloc(n * dim, sizeof(double));
       /* shift set to the beginning */
       nz = 0;
       for (i = 0; i < n; i++){
@@ -1275,11 +1267,11 @@ static int make_map_internal(int exclude_random, int include_OK_points,
       for (i = nh; i < n; i++){
 	grouping[i] = 2;
       }
-      memcpy(*xcombined, xtemp, n*dim*sizeof(real));
+      memcpy(*xcombined, xtemp, n*dim*sizeof(double));
       *nrandom += n - nh;/* count everything except cluster HIGHLIGHT_SET as random */
       n = nh;
       if (Verbose) fprintf(stderr,"nh = %d\n",nh);
-      FREE(xtemp);
+      free(xtemp);
     }
   }
 
@@ -1288,19 +1280,20 @@ static int make_map_internal(int exclude_random, int include_OK_points,
 	       poly_point_map, country_graph);
 
   SparseMatrix_delete(E);
-  FREE(Tp);
-  FREE(xran);
-  if (grouping != grouping0) FREE(grouping);
-  if (x != x0) FREE(x);
+  free(Tp);
+  free(xran);
+  if (grouping != grouping0) free(grouping);
+  if (x != x0) free(x);
   return 0;
 }
 
-static void add_point(int *n, int igrp, real **x, int *nmax, real point[], int **groups){
+static void add_point(int *n, int igrp, double **x, int *nmax, double point[], int **groups){
 
   if (*n >= *nmax){
+    int old_nmax = *nmax;
     *nmax = MAX((int) 0.2*(*n), 20) + *n;
-    *x = REALLOC(*x, sizeof(real)*2*(*nmax));
-    *groups = REALLOC(*groups, sizeof(int)*(*nmax));
+    *x = gv_recalloc(*x, 2 * old_nmax, 2 * *nmax, sizeof(double));
+    *groups = gv_recalloc(*groups, old_nmax, *nmax, sizeof(int));
   }
 
   (*x)[(*n)*2] = point[0];
@@ -1309,7 +1302,7 @@ static void add_point(int *n, int igrp, real **x, int *nmax, real point[], int *
   (*n)++;
 }
 
-static void get_boundingbox(int n, int dim, real *x, real *width, real *bbox){
+static void get_boundingbox(int n, int dim, double *x, double *width, double *bbox){
   int i;
   bbox[0] = bbox[1] = x[0];
   bbox[2] = bbox[3] = x[1];
@@ -1323,12 +1316,12 @@ static void get_boundingbox(int n, int dim, real *x, real *width, real *bbox){
 }
 
 int make_map_from_rectangle_groups(int exclude_random, int include_OK_points,
-				   int n, int dim, real *x, real *sizes, 
-				   int *grouping, SparseMatrix graph0, real bounding_box_margin[], int *nrandom, int *nart, int nedgep, 
-				   real shore_depth_tol, real edge_bridge_tol,
-				   real **xcombined, int *nverts, real **x_poly, 
+				   int n, int dim, double *x, double *sizes, 
+				   int *grouping, SparseMatrix graph0, double bounding_box_margin[], int *nrandom, int *nart, int nedgep, 
+				   double shore_depth_tol, double edge_bridge_tol,
+				   double **xcombined, int *nverts, double **x_poly, 
 				   int *npolys, SparseMatrix *poly_lines, SparseMatrix *polys, int **polys_groups, SparseMatrix *poly_point_map, 
-				   SparseMatrix *country_graph, int highlight_cluster, int *flag){
+				   SparseMatrix *country_graph, int highlight_cluster){
 
   /* create a list of polygons from a list of rectangles in 2D. rectangles belong to groups. rectangles in the same group that are also close 
      geometrically will be in the same polygon describing the outline of the group. The main difference for this function and
@@ -1389,18 +1382,18 @@ int make_map_from_rectangle_groups(int exclude_random, int include_OK_points,
 
      
   */
-    real dist, avgdist;
-  real *X;
+    double dist, avgdist;
+  double *X;
   int N, nmax, i, j, k, igrp;
   int *groups, K = *nart;/* average number of points added per side of rectangle */
 
-  real avgsize[2],  avgsz, h[2], p1, p0;
-  real point[2];
+  double avgsize[2],  avgsz, h[2], p1, p0;
+  double point[2];
   int nadded[2];
   int res;
-  real delta[2];
+  double delta[2];
   SparseMatrix graph = graph0;
-  real bbox[4];
+  double bbox[4];
 
   if (K < 0){
     K = (int) 10/(1+n/400.);/* 0 if n > 3600*/
@@ -1419,7 +1412,7 @@ int make_map_from_rectangle_groups(int exclude_random, int include_OK_points,
   if (!sizes){
     return make_map_internal(exclude_random, include_OK_points, n, dim, x, grouping, graph, bounding_box_margin, nrandom, nedgep, 
 			    shore_depth_tol, xcombined, nverts, x_poly, 
-			     npolys, poly_lines, polys, polys_groups, poly_point_map, country_graph, highlight_cluster, flag);
+			     npolys, poly_lines, polys, polys_groups, poly_point_map, country_graph, highlight_cluster);
   } else {
 
     /* add artificial node due to node sizes */
@@ -1435,8 +1428,8 @@ int make_map_from_rectangle_groups(int exclude_random, int include_OK_points,
     if (Verbose) fprintf(stderr, "avgsize = {%f, %f}\n",avgsize[0], avgsize[1]);
 
     nmax = 2*n;
-    X = MALLOC(sizeof(real)*dim*(n+nmax));
-    groups = MALLOC(sizeof(int)*(n+nmax));
+    X = gv_calloc(dim * (n + nmax), sizeof(double));
+    groups = gv_calloc(n + nmax, sizeof(int));
     for (i = 0; i < n; i++) {
       groups[i] = grouping[i];
       for (j = 0; j < 2; j++){
@@ -1448,10 +1441,10 @@ int make_map_from_rectangle_groups(int exclude_random, int include_OK_points,
     if (shore_depth_tol < 0) {
       shore_depth_tol = -(shore_depth_tol)*avgsz;
     } else if (shore_depth_tol == 0){
-      real area;
+      double area;
       get_boundingbox(n, dim, x, sizes, bbox);
       area = (bbox[1] - bbox[0])*(bbox[3] - bbox[2]);
-      shore_depth_tol = sqrt(area/(real) n); 
+      shore_depth_tol = sqrt(area/(double) n); 
       if (Verbose) fprintf(stderr,"setting shore length ======%f\n",shore_depth_tol);
     } else {
     }
@@ -1528,7 +1521,7 @@ int make_map_from_rectangle_groups(int exclude_random, int include_OK_points,
       int *ia = graph->ia, *ja = graph->ja, nz = 0, jj;
       int KB;
 
-      graph = SparseMatrix_symmetrize(graph, TRUE);
+      graph = SparseMatrix_symmetrize(graph, true);
       ia = graph->ia; ja = graph->ja;
       dist=avgdist = 0.;
       for (i = 0; i < n; i++){
@@ -1583,10 +1576,10 @@ int make_map_from_rectangle_groups(int exclude_random, int include_OK_points,
 
     res = make_map_internal(exclude_random, include_OK_points, N, dim, X, groups, graph, bounding_box_margin, nrandom, nedgep, 
 			    shore_depth_tol, xcombined, nverts, x_poly, 
-			    npolys, poly_lines, polys, polys_groups, poly_point_map, country_graph, highlight_cluster, flag);
+			    npolys, poly_lines, polys, polys_groups, poly_point_map, country_graph, highlight_cluster);
     if (graph != graph0) SparseMatrix_delete(graph);
-    FREE(groups); 
-    FREE(X);
+    free(groups);
+    free(X);
   }
 
   return res;
