@@ -10,8 +10,6 @@
 #ifdef _WIN32
 #include <windows.h>
 #include <io.h>
-#else
-#include <unistd.h>
 #endif
 #include "viewport.h"
 #include "draw.h"
@@ -20,6 +18,7 @@
 #include "gui.h"
 #include "menucallbacks.h"
 #include <stddef.h>
+#include <stdbool.h>
 #include <string.h>
 #include "glcompui.h"
 #include "gltemplate.h"
@@ -29,6 +28,7 @@
 #include "arcball.h"
 #include "hotkeymap.h"
 #include "topviewfuncs.h"
+#include <cgraph/exit.h>
 #include <cgraph/strcasecmp.h>
 
 
@@ -40,11 +40,6 @@ GtkMessageDialog *Dlg;
 
 static void clear_viewport(ViewInfo * view)
 {
-    /*free topview if there is one */
-    if (view->activeGraph >= 0)
-    {
-	freeSmGraph(view->g[view->activeGraph],view->Topview);
-    }
     if (view->graphCount)
 	agclose(view->g[view->activeGraph]);
 }
@@ -76,20 +71,17 @@ static void *get_glut_font(int ind)
 
 }
 
-int close_graph(ViewInfo * view, int graphid)
-{
+void close_graph(ViewInfo *view) {
     if (view->activeGraph < 0)
-	return 1;
+	return;
     clear_viewport(view);
-    return 1;
-
 }
 
 char *get_attribute_value(char *attr, ViewInfo * view, Agraph_t * g)
 {
     char *buf;
     buf = agget(g, attr);
-    if ((!buf) || (*buf == '\0'))
+    if (!buf || *buf == '\0')
 	buf = agget(view->systemGraphs.def_attrs, attr);
     return buf;
 }
@@ -241,6 +233,8 @@ void set_viewport_settings_from_template(ViewInfo * view, Agraph_t * g)
 
 static gboolean gl_main_expose(gpointer data)
 {
+    (void)data;
+
     if (view->activeGraph >= 0) {
 	if (view->Topview->fisheyeParams.animate == 1)
 	    expose_event(view->drawing_area, NULL, NULL);
@@ -251,10 +245,7 @@ static gboolean gl_main_expose(gpointer data)
 
 static void get_data_dir(void)
 {
-    if (view->template_file) {
-	free(view->template_file);
-    }
-
+    free(view->template_file);
     view->template_file = strdup(smyrnaPath("template.dot"));
 }
 
@@ -269,7 +260,7 @@ void init_viewport(ViewInfo * view)
 	fprintf(stderr,
 		"default attributes template graph file \"%s\" not found\n",
 		view->template_file);
-	exit(-1);
+	graphviz_exit(-1);
     } 
     view->systemGraphs.def_attrs = agread(input_file, NULL);
     fclose (input_file);
@@ -278,21 +269,21 @@ void init_viewport(ViewInfo * view)
 	fprintf(stderr,
 		"could not load default attributes template graph file \"%s\"\n",
 		view->template_file);
-	exit(-1);
+	graphviz_exit(-1);
     }
     if (!path)
 	path = smyrnaPath("attr_widgets.dot");
     input_file2 = fopen(path, "rb");
     if (!input_file2) {
 	fprintf(stderr,	"default attributes template graph file \"%s\" not found\n",smyrnaPath("attr_widgets.dot"));
-	exit(-1);
+	graphviz_exit(-1);
 
     }
     view->systemGraphs.attrs_widgets = agread(input_file2, NULL);
     fclose (input_file2);
-    if (!(view->systemGraphs.attrs_widgets )) {
+    if (!view->systemGraphs.attrs_widgets) {
 	fprintf(stderr,"could not load default attribute widgets graph file \"%s\"\n",smyrnaPath("attr_widgets.dot"));
-	exit(-1);
+	graphviz_exit(-1);
     }
     //init graphs
     view->g = NULL;		//no graph, gl screen should check it
@@ -389,14 +380,12 @@ void init_viewport(ViewInfo * view)
     view->active_frame = 0;
     view->total_frames = 1500;
     /*add a call back to the main() */
-    g_timeout_add_full((gint) G_PRIORITY_DEFAULT, (guint) 100,
-		       gl_main_expose, NULL, NULL);
-    view->cameras = '\0';;
+    g_timeout_add_full(G_PRIORITY_DEFAULT, 100u, gl_main_expose, NULL, NULL);
+    view->cameras = NULL;
     view->camera_count = 0;
     view->active_camera = -1;
     set_viewport_settings_from_template(view, view->systemGraphs.def_attrs);
-    view->Topview->Graphdata.GraphFileName = (char *) 0;
-    view->Topview->Graphdata.Modified = 0;
+    view->Topview->Graphdata.GraphFileName = NULL;
     view->colschms = NULL;
     view->arcball = NEW(ArcBall_t);
     view->keymap.down=0;
@@ -404,7 +393,6 @@ void init_viewport(ViewInfo * view)
     view->refresh.color=1;
     view->refresh.pos=1;
     view->refresh.selection=1;
-    view->refresh.visibility=1;
     view->refresh.nodesize=1;
     view->edgerendertype=0;
     if(view->guiMode!=GUI_FULLSCREEN)
@@ -489,7 +477,7 @@ void updateRecord (Agraph_t* g)
 static void
 graphRecord (Agraph_t* g)
 {
-    agbindrec(g, "graphRec", sizeof(graphRec), 1);
+    agbindrec(g, "graphRec", sizeof(graphRec), true);
 
     GG_nodelabelcolor(g) = agattr (g, AGRAPH, "nodelabelcolor", 0);
     GG_edgelabelcolor(g) = agattr (g, AGRAPH, "edgelabelcolor", 0);
@@ -502,15 +490,14 @@ graphRecord (Agraph_t* g)
     updateRecord (g);
 }
 
-void refreshViewport(int doClear)
+void refreshViewport(void)
 {
     Agraph_t *graph = view->g[view->activeGraph];
     view->refresh.color=1;
     view->refresh.nodesize=1;
     view->refresh.pos=1;
     view->refresh.selection=1;
-    view->refresh.visibility=1;
-    load_settings_from_graph(graph);
+    load_settings_from_graph();
 
     if(view->guiMode!=GUI_FULLSCREEN)
 	update_graph_from_settings(graph);
@@ -521,10 +508,10 @@ void refreshViewport(int doClear)
     expose_event(view->drawing_area, NULL, NULL);
 }
 
-static void activate(int id, int doClear)
+static void activate(int id)
 {
     view->activeGraph = id;
-    refreshViewport(doClear);
+    refreshViewport();
 }
 
 int add_graph_to_viewport(Agraph_t * graph, char *id)
@@ -537,7 +524,7 @@ int add_graph_to_viewport(Agraph_t * graph, char *id)
 
 	gtk_combo_box_append_text(view->graphComboBox, id);
 	gtk_combo_box_set_active(view->graphComboBox,view->graphCount-1);
-	activate(view->graphCount - 1, 0);
+	activate(view->graphCount - 1);
 	return 1;
     } else {
 	return 0;
@@ -546,10 +533,10 @@ int add_graph_to_viewport(Agraph_t * graph, char *id)
 }
 void switch_graph(int graphId)
 {
-    if ((graphId >= view->graphCount) || (graphId < 0))
+    if (graphId >= view->graphCount || graphId < 0)
 	return;			/*wrong entry */
     else
-	activate(graphId, 0);
+	activate(graphId);
 }
 
 /* save_graph_with_file_name:
@@ -609,8 +596,7 @@ int save_as_graph(void)
 {
     //check if there is an active graph
     if (view->activeGraph > -1) {
-	GtkWidget *dialog;
-	dialog = gtk_file_chooser_dialog_new("Save File",
+	GtkWidget *dialog = gtk_file_chooser_dialog_new("Save File",
 					     NULL,
 					     GTK_FILE_CHOOSER_ACTION_SAVE,
 					     GTK_STOCK_CANCEL,
@@ -620,8 +606,7 @@ int save_as_graph(void)
 	gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER
 						       (dialog), TRUE);
 	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
-	    char *filename;
-	    filename =
+	    char *filename =
 		gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
 	    save_graph_with_file_name(view->g[view->activeGraph],
 				      filename);
@@ -644,7 +629,7 @@ void glexpose(void)
 
 static float interpol(float minv, float maxv, float minc, float maxc, float x)
 {
-    return ((x - minv) * (maxc - minc) / (maxv - minv) + minc);
+    return (x - minv) * (maxc - minc) / (maxv - minv) + minc;
 }
 
 void getcolorfromschema(colorschemaset * sc, float l, float maxl,
@@ -739,7 +724,7 @@ static colorschemaset *create_color_theme(int themeid)
 {
     colorschemaset *s;
 
-    if ((themeid < 0) || (NUM_SCHEMES <= themeid)) {
+    if (themeid < 0 || (int)NUM_SCHEMES <= themeid) {
 	fprintf (stderr, "colorschemaset: illegal themeid %d\n", themeid);
 	return view->colschms;
     }
