@@ -18,12 +18,15 @@
 #include <ast/ast.h>
 #include <gvpr/compile.h>
 #include <ast/sfstr.h>
+#include <limits.h>
 #include <stddef.h>
 #include <string.h>
 #include <stdio.h>
 #include <ctype.h>
 #include <cgraph/agxbuf.h>
+#include <cgraph/likely.h>
 #include <cgraph/strcasecmp.h>
+#include <cgraph/unreachable.h>
 
 #define KINDS(p) ((AGTYPE(p) == AGRAPH) ? "graph" : (AGTYPE(p) == AGNODE) ? "node" : "edge")
 
@@ -63,15 +66,17 @@ int indexOf(char *s1, char *s2)
 /* rindexOf:
  * Return index of rightmost string s2 in string s1, or -1
  */
-int rindexOf(char *s1, char *s2)
+long rindexOf(char *s1, char *s2)
 {
     char c1 = *s2;
     char *p;
     size_t len1 = strlen(s1);
     size_t len2 = strlen(s2);
 
-    if (c1 == '\0')
-	return (len1);
+    if (c1 == '\0') {
+	assert(len1 <= LONG_MAX);
+	return (long)len1;
+    }
     if (len2 > len1)
 	return -1;
     p = s1 + (len1 - len2);
@@ -197,6 +202,8 @@ Agobj_t *copy(Agraph_t * g, Agobj_t * obj)
 	name = agnameof (AGMKOUT(e));
 	nobj = (Agobj_t *) openEdge(g, t, h, name);
 	break;
+    default:
+	UNREACHABLE();
     }
     if (nobj)
 	copyAttr(obj, nobj);
@@ -275,6 +282,9 @@ static Agraph_t *cloneSubg(Agraph_t * tgt, Agraph_t * g, Dt_t* emap)
 
 static int cmppair(Dt_t * d, Agedge_t** key1, Agedge_t** key2, Dtdisc_t * disc)
 {
+    (void)d;
+    (void)disc;
+
     if (*key1 > *key2) return 1;
     else if (*key1 < *key2) return -1;
     else return 0;
@@ -354,7 +364,7 @@ Agraph_t *cloneG(Agraph_t * g, char* name)
     return ng;
 }
 
-/* clone:
+/* cloneO:
  * Create new object of type AGTYPE(obj) with all of its
  * attributes and substructure.
  * If obj is an edge, end nodes are cloned if necessary.
@@ -362,7 +372,7 @@ Agraph_t *cloneG(Agraph_t * g, char* name)
  * graph. Otherwise, create a clone subgraph of g.
  * Assume obj != NULL.
  */
-Agobj_t *clone(Agraph_t * g, Agobj_t * obj)
+Agobj_t *cloneO(Agraph_t * g, Agobj_t * obj)
 {
     Agobj_t *nobj = 0;
     Agedge_t *e;
@@ -397,13 +407,15 @@ Agobj_t *clone(Agraph_t * g, Agobj_t * obj)
     case AGINEDGE:
     case AGOUTEDGE:
 	e = (Agedge_t *) obj;
-	t = (Agnode_t *) clone(g, OBJ(agtail(e)));
-	h = (Agnode_t *) clone(g, OBJ(aghead(e)));
+	t = (Agnode_t *) cloneO(g, OBJ(agtail(e)));
+	h = (Agnode_t *) cloneO(g, OBJ(aghead(e)));
 	name = agnameof (AGMKOUT(e));
 	nobj = (Agobj_t *) openEdge(g, t, h, name);
 	if (nobj)
 	    copyAttr(obj, nobj);
 	break;
+    default:
+	UNREACHABLE();
     }
 
     return nobj;
@@ -594,7 +606,7 @@ int writeFile(Agraph_t * g, char *f, Agiodisc_t* io)
 	exerror("NULL string passed to writeG");
 	return 1;
     }
-    fp = sfopen(0, f, "w");
+    fp = sfopen(f, "w");
     if (!fp) {
 	exwarn("Could not open %s for writing in writeG", f);
 	return 1;
@@ -617,7 +629,7 @@ Agraph_t *readFile(char *f)
 	exerror("NULL string passed to readG");
 	return 0;
     }
-    fp = sfopen(0, f, "r");
+    fp = sfopen(f, "r");
     if (!fp) {
 	exwarn("Could not open %s for reading in readG", f);
 	return 0;
@@ -632,7 +644,8 @@ int fwriteFile(Expr_t * ex, Agraph_t * g, int fd, Agiodisc_t* io)
 {
     Sfio_t *sp;
 
-    if (fd < 0 || fd >= elementsof(ex->file)
+    if (fd < 0 ||
+        (elementsof(ex->file) <= INT_MAX && fd >= (int)elementsof(ex->file))
 	|| !((sp = ex->file[fd]))) {
 	exerror("fwriteG: %d: invalid descriptor", fd);
 	return 0;
@@ -644,7 +657,8 @@ Agraph_t *freadFile(Expr_t * ex, int fd)
 {
     Sfio_t *sp;
 
-    if (fd < 0 || fd >= elementsof(ex->file)
+    if (fd < 0 ||
+        (elementsof(ex->file) <= INT_MAX && fd >= (int)elementsof(ex->file))
 	|| !((sp = ex->file[fd]))) {
 	exerror("freadG: %d: invalid descriptor", fd);
 	return 0;
@@ -664,7 +678,7 @@ int openFile(Expr_t * ex, const char *fname, const char *mode)
 	exerror("openF: no available descriptors");
 	return -1;
     }
-    ex->file[idx] = sfopen(0, fname, mode);
+    ex->file[idx] = sfopen(fname, mode);
     if (ex->file[idx])
 	return idx;
     else
@@ -697,18 +711,18 @@ char *readLine(Expr_t * ex, int fd)
 {
     Sfio_t *sp;
     int c;
-    agxbuf tmps;
     char *line;
 
     if (fd < 0 || fd >= elementsof(ex->file) || !((sp = ex->file[fd]))) {
 	exerror("readL: %d: invalid descriptor", fd);
 	return "";
     }
-    agxbinit(&tmps, 0, NULL);
+
+    agxbuf tmps = {0};
     while ((c = sfgetc(sp)) > 0 && c != '\n')
-	agxbputc(&tmps, c);
+	agxbputc(&tmps, (char)c);
     if (c == '\n')
-	agxbputc(&tmps, c);
+	agxbputc(&tmps, (char)c);
     line = exstring(ex, agxbuse(&tmps));
     agxbfree(&tmps);
     return line;
@@ -749,27 +763,39 @@ int compare(Agobj_t * l, Agobj_t * r)
 /* toLower:
  * Convert characters to lowercase
  */
-char *toLower(Expr_t * pgm, char *s, Sfio_t* tmps)
-{
-    int c;
+char *toLower(Expr_t *pgm, char *src) {
 
-    while ((c = *s++))
-	sfputc (tmps, tolower (c));
+  const size_t len = strlen(src);
+  char *dst = exstralloc(pgm, len + 1);
+  if (UNLIKELY(dst == NULL)) {
+    return NULL;
+  }
 
-    return exstring(pgm, sfstruse(tmps));
+  for (size_t i = 0; i < len; ++i) {
+    dst[i] = (char)tolower((int)src[i]);
+  }
+
+  dst[len] = '\0';
+  return dst;
 }
 
 /* toUpper:
  * Convert characters to uppercase
  */
-char *toUpper(Expr_t * pgm, char *s, Sfio_t* tmps)
-{
-    int c;
+char *toUpper(Expr_t *pgm, char *src) {
 
-    while ((c = *s++))
-	sfputc (tmps, toupper (c));
+  const size_t len = strlen(src);
+  char *dst = exstralloc(pgm, len + 1);
+  if (UNLIKELY(dst == NULL)) {
+    return NULL;
+  }
 
-    return exstring(pgm, sfstruse(tmps));
+  for (size_t i = 0; i < len; ++i) {
+    dst[i] = (char)toupper((int)src[i]);
+  }
+
+  dst[len] = '\0';
+  return dst;
 }
 
 /* toHtml:
@@ -909,29 +935,27 @@ static int colorcmpf(const void *p0, const void *p1)
 
 static char *canontoken(char *str)
 {
-    static unsigned char *canon;
+    static char *canon;
     static size_t allocated;
-    unsigned char c, *p, *q;
+    char c, *p, *q;
     size_t len;
 
-    p = (unsigned char *) str;
+    p = str;
     len = strlen(str);
     if (len >= allocated) {
 	allocated = len + 1 + 10;
-	canon = newof(canon, unsigned char, allocated, 0);
+	canon = newof(canon, char, allocated, 0);
 	if (!canon)
 	    return NULL;
     }
     q = canon;
     while ((c = *p++)) {
-	/* if (isalnum(c) == FALSE) */
-	    /* continue; */
-	if (isupper(c))
-	    c = tolower(c);
+	if (isupper((int)c))
+	    c = (char)tolower((int)c);
 	*q++ = c;
     }
     *q = '\0';
-    return (char*)canon;
+    return canon;
 }
 
 /* fullColor:
@@ -1252,8 +1276,7 @@ int colorxlate(char *str, gvcolor_t * color, color_type_t target_type)
 /* colorx:
  * RGB, RGBA, HSV, HSVA, CMYK
  */
-char *colorx (Expr_t* ex, char* incolor, char* fmt, Sfio_t* fp)
-{
+char *colorx(Expr_t *ex, char *incolor, char *fmt) {
     gvcolor_t color = {{{0}},0};
     color_type_t type;
     int rc;
@@ -1286,28 +1309,33 @@ char *colorx (Expr_t* ex, char* incolor, char* fmt, Sfio_t* fp)
     if (rc != COLOR_OK)
 	return "";
 
+    agxbuf fp = {0};
+    agxbinit(&fp, 0, NULL);
+
     switch (type) {
     case HSVA_DOUBLE :
-	sfprintf (fp, "%.03f %.03f %.03f", 
+	agxbprint(&fp, "%.03f %.03f %.03f",
 	    color.u.HSVA[0], color.u.HSVA[1], color.u.HSVA[2]);
 	if (alpha)
-	    sfprintf (fp, " %.03f", color.u.HSVA[3]);
+	    agxbprint(&fp, " %.03f", color.u.HSVA[3]);
 	break;
     case RGBA_BYTE :
-	sfprintf (fp, "#%02x%02x%02x", 
+	agxbprint(&fp, "#%02x%02x%02x",
 	    color.u.rgba[0], color.u.rgba[1], color.u.rgba[2]);
 	if (alpha)
-	    sfprintf (fp, "%02x", color.u.rgba[3]);
+	    agxbprint(&fp, "%02x", color.u.rgba[3]);
 	break;
     case CMYK_BYTE :
-	sfprintf (fp, "#%02x%02x%02x%02x", 
+	agxbprint(&fp, "#%02x%02x%02x%02x",
 	    color.u.cmyk[0], color.u.cmyk[1], color.u.cmyk[2], color.u.cmyk[3]);
 	break;
     default :
 	break;
     }
 
-    return exstring(ex, sfstruse(fp));
+    char *result = exstring(ex, agxbuse(&fp));
+    agxbfree(&fp);
+    return result;
 }
 
 #ifndef _WIN32
