@@ -19,6 +19,7 @@
 #include <unistd.h>
 #include <gvpr/compile.h>
 #include <assert.h>
+#include <cgraph/agxbuf.h>
 #include <cgraph/cgraph.h>
 #include <cgraph/exit.h>
 #include <cgraph/itos.h>
@@ -88,8 +89,7 @@ static Agdisc_t gprDisc = { &AgMemDisc, &AgIdDisc, &gprIoDisc };
  * Return name of object. 
  * Assumes obj !=  NULL
  */
-static char *nameOf(Expr_t * ex, Agobj_t * obj, Sfio_t* tmps)
-{
+static char *nameOf(Expr_t *ex, Agobj_t *obj, agxbuf *tmps) {
     char *s;
     char *key;
     Agedge_t *e;
@@ -102,18 +102,18 @@ static char *nameOf(Expr_t * ex, Agobj_t * obj, Sfio_t* tmps)
     default:			/* edge */
 	e = (Agedge_t *) obj;
 	key = agnameof(AGMKOUT(e));
-	sfputr(tmps, agnameof(AGTAIL(e)), -1);
+	agxbput(tmps, agnameof(AGTAIL(e)));
 	if (agisdirected(agraphof(e)))
-	    sfputr(tmps, "->", -1);
+	    agxbput(tmps, "->");
 	else
-	    sfputr(tmps, "--", -1);
-	sfputr(tmps, agnameof(AGHEAD(e)), -1);
+	    agxbput(tmps, "--");
+	agxbput(tmps, agnameof(AGHEAD(e)));
 	if (key && *key) {
-	    sfputc(tmps, '[');
-	    sfputr(tmps, key, -1);
-	    sfputc(tmps, ']');
+	    agxbputc(tmps, '[');
+	    agxbput(tmps, key);
+	    agxbputc(tmps, ']');
 	}
-	s = exstring(ex, sfstruse(tmps));
+	s = exstring(ex, agxbuse(tmps));
 	break;
     }
     return s;
@@ -264,12 +264,10 @@ static Agdesc_t xargs(char *args)
 /* deparse:
  * Recreate string representation of expression involving
  * a reference and a symbol.
- * The parameter sf must be a string stream.
  */
-static char *deparse(Expr_t * ex, Exnode_t * n, Sfio_t * sf)
-{
-    exdump(ex, n, sf);
-    return sfstruse(sf);
+static char *deparse(Expr_t *ex, Exnode_t *n, agxbuf *xb) {
+  exdump(ex, n, xb);
+  return xb->buf;
 }
 
 /* deref:
@@ -289,9 +287,10 @@ static Agobj_t *deref(Expr_t * pgm, Exnode_t * x, Exref_t * ref,
 	ptr = int2ptr(x->data.variable.dyna->data.variable.dyna->data.
 		    constant.value.integer);
 	if (!ptr) {
+	    agxbuf xb = {0};
 	    exerror("null reference %s in expression %s.%s",
-		  ref->symbol->name, ref->symbol->name, deparse(pgm, x,
-								state->tmp));
+		  ref->symbol->name, ref->symbol->name, deparse(pgm, x, &xb));
+	    agxbfree(&xb);
 	    return ptr;
 	} else
 	    return deref(pgm, x, ref->next, (Agobj_t *) ptr, state);
@@ -422,9 +421,7 @@ kindOf (Agobj_t* objp)
  * Apply symbol to get field value of objp
  * Assume objp != NULL
  */
-static int lookup(Expr_t * pgm, Agobj_t * objp, Exid_t * sym, Extype_t * v,
-  Gpr_t *state)
-{
+static int lookup(Expr_t *pgm, Agobj_t *objp, Exid_t *sym, Extype_t * v) {
     if (sym->lex == ID) {
 	switch (sym->index) {
 	case M_head:
@@ -443,9 +440,13 @@ static int lookup(Expr_t * pgm, Agobj_t * objp, Exid_t * sym, Extype_t * v,
 		return -1;
 	    }
 	    break;
-	case M_name:
-	    v->string = nameOf(pgm, objp, state->tmp);
+	case M_name: {
+	    agxbuf tmp = {0};
+	    agxbinit(&tmp, 0, NULL);
+	    v->string = nameOf(pgm, objp, &tmp);
+	    agxbfree(&tmp);
 	    break;
+	}
 	case M_indegree:
 	    if (AGTYPE(objp) == AGNODE)
 		v->integer = agdegree(agroot(objp), (Agnode_t *) objp, 1, 0);
@@ -540,7 +541,12 @@ static int lookup(Expr_t * pgm, Agobj_t * objp, Exid_t * sym, Extype_t * v,
 	Agsym_t *gsym = agattrsym(objp, sym->name);
 	if (!gsym) {
 	    gsym = agattr(agroot(agraphof(objp)), AGTYPE(objp), sym->name, "");
-	    error(ERROR_WARNING, "Using value of uninitialized %s attribute \"%s\" of \"%s\"", kindOf (objp), sym->name, nameOf(pgm, objp, state->tmp));
+	    agxbuf tmp = {0};
+	    agxbinit(&tmp, 0, NULL);
+	    error(ERROR_WARNING,
+	          "Using value of uninitialized %s attribute \"%s\" of \"%s\"",
+	          kindOf (objp), sym->name, nameOf(pgm, objp, &tmp));
+	    agxbfree(&tmp);
 	}
 	v->string = agxget(objp, gsym);
     }
@@ -1144,7 +1150,7 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 		error(ERROR_WARNING, "NULL object passed to clone()");
 		v.integer = 0;
 	    } else
-		v.integer = ptr2int(clone(gp, objp));
+		v.integer = ptr2int(cloneO(gp, objp));
 	    break;
 	case F_cloneG:
 	    gp = int2ptr(args[0].integer);
@@ -1332,7 +1338,12 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 		else {
 		    if (!gsym) {
 	    		gsym = agattr(agroot(agraphof(objp)), AGTYPE(objp), name, "");
-	    		error(ERROR_WARNING, "Using value of %s uninitialized attribute \"%s\" of \"%s\" in aget()", kindOf (objp), name, nameOf(pgm, objp, state->tmp));
+	    		agxbuf tmp = {0};
+	    		agxbinit(&tmp, 0, NULL);
+	    		error(ERROR_WARNING,
+	    		      "Using value of %s uninitialized attribute \"%s\" of \"%s\" in aget()",
+	    		      kindOf (objp), name, nameOf(pgm, objp, &tmp));
+	    		agxbfree(&tmp);
 		    }
 		    v.string = agxget(objp, gsym);
         	}
@@ -1446,10 +1457,10 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 	    }
 	    break;
 	case F_tolower:
-	    v.string = toLower(pgm, args[0].string, state->tmp);
+	    v.string = toLower(pgm, args[0].string);
 	    break;
 	case F_colorx:
-	    v.string = colorx(pgm, args[0].string, args[1].string, state->tmp);
+	    v.string = colorx(pgm, args[0].string, args[1].string);
 	    break;
 	case F_strcmp:
 	    if (args[0].string) {
@@ -1463,7 +1474,7 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 		v.integer = 0; 
 	    break;
 	case F_toupper:
-	    v.string = toUpper(pgm, args[0].string, state->tmp);
+	    v.string = toUpper(pgm, args[0].string);
 	    break;
 	case F_xof:
 	    v.string = xyOf(pgm, args[0].string, true);
@@ -1517,9 +1528,11 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
     state = env;
     if (ref) {
 	objp = deref(pgm, node, ref, 0, state);
-	if (!objp)
-	    exerror("null reference in expression %s",
-		  deparse(pgm, node, state->tmp));
+	if (!objp) {
+	    agxbuf xb = {0};
+	    exerror("null reference in expression %s", deparse(pgm, node, &xb));
+	    agxbfree(&xb);
+	}
     } else if (sym->lex == ID && sym->index <= LAST_V) {
 	switch (sym->index) {
 	case V_this:
@@ -1563,14 +1576,18 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
     } else {
 	objp = state->curobj;
 	if (!objp) {
+	    agxbuf xb = {0};
 	    exerror("current object $ not defined as reference for %s",
-		  deparse(pgm, node, state->tmp));
+		  deparse(pgm, node, &xb));
+	    agxbfree(&xb);
 	}
     }
 
     if (objp) {
-	if (lookup(pgm, objp, sym, &v, state)) {
-	    exerror("in expression %s", deparse(pgm, node, state->tmp));
+	if (lookup(pgm, objp, sym, &v)) {
+	    agxbuf xb = {0};
+	    exerror("in expression %s", deparse(pgm, node, &xb));
+	    agxbfree(&xb);
 	    v.integer = 0;
 	}
     }
@@ -1582,8 +1599,7 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 
 #define MINTYPE (LAST_M+1)	/* First type occurs after last M_ */
 
-static char *typeName(Expr_t * pg, int op)
-{
+static char *typeName(int op) {
     return typenames[op - MINTYPE];
 }
 
@@ -1594,23 +1610,20 @@ static char *typeName(Expr_t * pg, int op)
  */
 static int
 setval(Expr_t * pgm, Exnode_t * x, Exid_t * sym, Exref_t * ref,
-       void *env, int elt, Extype_t v, Exdisc_t * disc)
+       void *env, Extype_t v)
 {
-    (void)elt;
-    (void)disc;
-
     Gpr_t *state;
     Agobj_t *objp;
     Agnode_t *np;
-    int iv;
     int rv = 0;
 
     state = env;
     if (ref) {
 	objp = deref(pgm, x, ref, 0, state);
 	if (!objp) {
-	    exerror("in expression %s.%s",
-		  ref->symbol->name, deparse(pgm, x, state->tmp));
+	    agxbuf xb = {0};
+	    exerror("in expression %s.%s", ref->symbol->name, deparse(pgm, x, &xb));
+	    agxbfree(&xb);
 	    return -1;
 	}
     } else if (MINNAME <= sym->index && sym->index <= MAXNAME) {
@@ -1618,14 +1631,15 @@ setval(Expr_t * pgm, Exnode_t * x, Exid_t * sym, Exref_t * ref,
 	case V_outgraph:
 	    state->outgraph = int2ptr(v.integer);
 	    break;
-	case V_travtype:
-	    iv = v.integer;
+	case V_travtype: {
+	    long long iv = v.integer;
 	    if (validTVT(v.integer))
 		state->tvt = (trav_type) iv;
 	    else
-		error(1, "unexpected value %d assigned to %s : ignored",
-		      iv, typeName(pgm, T_tvtyp));
+		error(1, "unexpected value %lld assigned to %s : ignored",
+		      iv, typeName(T_tvtyp));
 	    break;
+	}
 	case V_travroot:
 	    np = int2ptr(v.integer);
 	    if (!np || agroot(np) == state->curgraph)
@@ -1660,8 +1674,10 @@ setval(Expr_t * pgm, Exnode_t * x, Exid_t * sym, Exref_t * ref,
     } else {
 	objp = state->curobj;
 	if (!objp) {
+	    agxbuf xb = {0};
 	    exerror("current object $ undefined in expression %s",
-		  deparse(pgm, x, state->tmp));
+		  deparse(pgm, x, &xb));
+	    agxbfree(&xb);
 	    return -1;
 	}
     }
@@ -1793,11 +1809,8 @@ static tctype typeChkExp(Exref_t * ref, Exid_t * sym)
  * Type check expressions; return value unused.
  */
 static Extype_t
-refval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
-       char *str, int elt, Exdisc_t * disc)
+refval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref)
 {
-    (void)str;
-    (void)elt;
 
     Extype_t v;
     if (sym->lex == CONSTANT) {
@@ -1850,9 +1863,9 @@ refval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 	}
     } else {
 	if (!typeChkExp(ref, sym)) {
-	    Gpr_t *state = disc->user;
-	    exerror("type error using %s",
-		    deparse(pgm, node, state->tmp));
+	    agxbuf xb = {0};
+	    exerror("type error using %s", deparse(pgm, node, &xb));
+	    agxbfree(&xb);
 	}
 	v = exzero(node->type);
     }
@@ -1866,13 +1879,7 @@ refval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
  * Return -1 if operation cannot be done, 0 otherwise.
  * If arg is > 0, operation unnecessary; just report possibility.
  */
-static int
-binary(Expr_t * pg, Exnode_t * l, Exnode_t * ex, Exnode_t * r, int arg,
-       Exdisc_t * disc)
-{
-    (void)pg;
-    (void)disc;
-
+static int binary(Exnode_t * l, Exnode_t * ex, Exnode_t * r, int arg) {
     Agobj_t *lobjp;
     Agobj_t *robjp;
     int ret = -1;
@@ -1885,50 +1892,49 @@ binary(Expr_t * pg, Exnode_t * l, Exnode_t * ex, Exnode_t * r, int arg,
 	return -1;
 
     if (l->type == T_tvtyp) {
-	int li, ri;
 
 	if (!r)
 	    return -1;		/* Assume libexpr handled unary */
 	if (r->type != T_tvtyp)
 	    return -1;
 
-	li = l->data.constant.value.integer;
-	ri = r->data.constant.value.integer;
+	long long li = l->data.constant.value.integer;
+	long long ri = r->data.constant.value.integer;
 	switch (ex->op) {
 	case EQ:
 	    if (arg)
 		return 0;
-	    l->data.constant.value.integer = (li == ri);
+	    l->data.constant.value.integer = li == ri;
 	    ret = 0;
 	    break;
 	case NE:
 	    if (arg)
 		return 0;
-	    l->data.constant.value.integer = (li != ri);
+	    l->data.constant.value.integer = li != ri;
 	    ret = 0;
 	    break;
 	case '<':
 	    if (arg)
 		return 0;
-	    l->data.constant.value.integer = (li < ri);
+	    l->data.constant.value.integer = li < ri;
 	    ret = 0;
 	    break;
 	case LE:
 	    if (arg)
 		return 0;
-	    l->data.constant.value.integer = (li <= ri);
+	    l->data.constant.value.integer = li <= ri;
 	    ret = 0;
 	    break;
 	case GE:
 	    if (arg)
 		return 0;
-	    l->data.constant.value.integer = (li >= ri);
+	    l->data.constant.value.integer = li >= ri;
 	    ret = 0;
 	    break;
 	case '>':
 	    if (arg)
 		return 0;
-	    l->data.constant.value.integer = (li > ri);
+	    l->data.constant.value.integer = li > ri;
 	    ret = 0;
 	    break;
 	}
@@ -1959,25 +1965,25 @@ binary(Expr_t * pg, Exnode_t * l, Exnode_t * ex, Exnode_t * r, int arg,
     case '<':
 	if (arg)
 	    return 0;
-	l->data.constant.value.integer = (compare(lobjp, robjp) < 0);
+	l->data.constant.value.integer = compare(lobjp, robjp) < 0;
 	ret = 0;
 	break;
     case LE:
 	if (arg)
 	    return 0;
-	l->data.constant.value.integer = (compare(lobjp, robjp) <= 0);
+	l->data.constant.value.integer = compare(lobjp, robjp) <= 0;
 	ret = 0;
 	break;
     case GE:
 	if (arg)
 	    return 0;
-	l->data.constant.value.integer = (compare(lobjp, robjp) >= 0);
+	l->data.constant.value.integer = compare(lobjp, robjp) >= 0;
 	ret = 0;
 	break;
     case '>':
 	if (arg)
 	    return 0;
-	l->data.constant.value.integer = (compare(lobjp, robjp) > 0);
+	l->data.constant.value.integer = compare(lobjp, robjp) > 0;
 	ret = 0;
 	break;
     }
@@ -2030,9 +2036,7 @@ strToTvtype (char* s)
 
 /* tvtypeToStr:
  */
-static char*
-tvtypeToStr (int v)
-{
+static char *tvtypeToStr(long long v) {
     char* s = 0;
 
     switch (v) {
@@ -2076,8 +2080,7 @@ tvtypeToStr (int v)
 	s = "TV_prepostrev";
 	break;
     default:
-	exerror("Unexpected value %d for type tvtype_t",
-	    v);
+	exerror("Unexpected value %lld for type tvtype_t", v);
 	break;
     }
     return s;
@@ -2089,8 +2092,7 @@ tvtypeToStr (int v)
  * Return -1 if conversion cannot be done, 0 otherwise.
  * If arg is > 0, conversion unnecessary; just report possibility.
  */
-static int stringOf(Expr_t * prog, Exnode_t * x, int arg, Exdisc_t* disc)
-{
+static int stringOf(Expr_t *prog, Exnode_t *x, int arg) {
     Agobj_t *objp;
     int rv = 0;
 
@@ -2104,13 +2106,14 @@ static int stringOf(Expr_t * prog, Exnode_t * x, int arg, Exdisc_t* disc)
     } else {
 	objp = int2ptr(x->data.constant.value.integer);
 	if (!objp) {
-	    exerror("cannot generate name for NULL %s",
-		    typeName(prog, x->type));
+	    exerror("cannot generate name for NULL %s", typeName(x->type));
 	    rv = -1;
 	}
 	else {
-	    Gpr_t* state = disc->user;
-	    x->data.constant.value.string = nameOf(prog, objp, state->tmp);
+	    agxbuf tmp = {0};
+	    agxbinit(&tmp, 0, NULL);
+	    x->data.constant.value.string = nameOf(prog, objp, &tmp);
+	    agxbfree(&tmp);
 	}
     }
     x->type = STRING;
@@ -2123,14 +2126,7 @@ static int stringOf(Expr_t * prog, Exnode_t * x, int arg, Exdisc_t* disc)
  * If arg is > 0, conversion unnecessary; just report possibility.
  * In particular, assume x != 0 if arg == 0.
  */
-static int
-convert(Expr_t * prog, Exnode_t * x, int type,
-	Exid_t * xref, int arg, Exdisc_t * disc)
-{
-    (void)prog;
-    (void)xref;
-    (void)disc;
-
+static int convert(Exnode_t *x, int type, int arg) {
     Agobj_t *objp;
     int ret = -1;
 
@@ -2207,11 +2203,7 @@ convert(Expr_t * prog, Exnode_t * x, int type,
  * Calculate unique key for object.
  * We use this to unify local copies of nodes and edges.
  */
-static Extype_t keyval(Expr_t * pgm, Extype_t v, int type, Exdisc_t * disc)
-{
-    (void)pgm;
-    (void)disc;
-
+static Extype_t keyval(Extype_t v, int type) {
     if (type <= T_obj) {
 	v.integer = AGID(int2ptr(v.integer));
     }
@@ -2221,16 +2213,7 @@ static Extype_t keyval(Expr_t * pgm, Extype_t v, int type, Exdisc_t * disc)
 /* matchval:
  * Pattern match strings.
  */
-static int
-matchval(Expr_t * pgm, Exnode_t * xstr, const char *str, Exnode_t * xpat,
-	 const char *pat, void *env, Exdisc_t * disc)
-{
-    (void)pgm;
-    (void)xstr;
-    (void)xpat;
-    (void)env;
-    (void)disc;
-
+static int matchval(const char *str, const char *pat) {
     return strgrpmatch(str, pat, NULL, 0,
 		       STR_MAXIMAL | STR_LEFT | STR_RIGHT);
 }
@@ -2295,17 +2278,17 @@ static Exnode_t *compile(Expr_t * prog, char *src, char *input, int line,
 
     /* create input stream */
     if (sfx) {
-	sf = sfopen(0, sfx, "rs");
+	sf = sfopen(sfx, "rs");
 	if (input) {
-	    prefix = sfopen(0, input, "rs");
+	    prefix = sfopen(input, "rs");
 	    sfstack(sf, prefix);
 	}
     } else
-	sf = sfopen(0, input, "rs");
+	sf = sfopen(input, "rs");
 
     /*  prefixing label if necessary */
     if (lbl) {
-	prefix = sfopen(0, 0, "sr+");
+	prefix = sfopen(0, "sr+");
 	sfprintf(prefix, "%s:\n", lbl);
 	sfseek(prefix, 0, 0);
 	sfstack(sf, prefix);
@@ -2314,7 +2297,7 @@ static Exnode_t *compile(Expr_t * prog, char *src, char *input, int line,
 
     if (!src)
 	src = "<command line>";
-    rv = excomp(prog, src, line, 0, sf);
+    rv = excomp(prog, src, line, sf);
     sfclose(sf);
 
     if (rv >= 0 && getErrorErrors() == 0)
