@@ -32,7 +32,13 @@ static int icompare(Dt_t * dt, void * v1, void * v2, Dtdisc_t * disc)
     (void)dt;
     (void)disc;
     int k1 = *((int *) v1), k2 = *((int *) v2);
-    return k1 - k2;
+    if (k1 < k2) {
+      return -1;
+    }
+    if (k1 > k2) {
+      return 1;
+    }
+    return 0;
 }
 
 static XLabels_t *xlnew(object_t * objs, int n_objs, xlabel_t * lbls,
@@ -267,7 +273,7 @@ static int getintrsxi(object_t * op, object_t * cp)
 
 /* record the intersecting objects label */
 static double
-recordointrsx(XLabels_t * xlp, object_t * op, object_t * cp, Rect_t * rp,
+recordointrsx(object_t * op, object_t * cp, Rect_t * rp,
 	      double a, object_t * intrsx[XLNBR])
 {
     int i = getintrsxi(op, cp);
@@ -300,7 +306,7 @@ recordointrsx(XLabels_t * xlp, object_t * op, object_t * cp, Rect_t * rp,
 
 /* record the intersecting label */
 static double
-recordlintrsx(XLabels_t * xlp, object_t * op, object_t * cp, Rect_t * rp,
+recordlintrsx(object_t * op, object_t * cp, Rect_t * rp,
 	      double a, object_t * intrsx[XLNBR])
 {
     int i = getintrsxi(op, cp);
@@ -369,7 +375,7 @@ xlintersections(XLabels_t * xlp, object_t * objp, object_t * intrsx[XLNBR])
 	objp2rect(cp, &srect);
 	a = aabbaabb(&rect, &srect);
 	if (a > 0.0) {
-	  ra = recordointrsx(xlp, objp, cp, &rect, a, intrsx);
+	  ra = recordointrsx(objp, cp, &rect, a, intrsx);
 	  bp.n++;
 	  bp.area += ra;
 	}
@@ -379,7 +385,7 @@ xlintersections(XLabels_t * xlp, object_t * objp, object_t * intrsx[XLNBR])
 	objplp2rect(cp, &srect);
 	a = aabbaabb(&rect, &srect);
 	if (a > 0.0) {
-	  ra = recordlintrsx(xlp, objp, cp, &rect, a, intrsx);
+	  ra = recordlintrsx(objp, cp, &rect, a, intrsx);
 	  bp.n++;
 	  bp.area += ra;
 	}
@@ -398,7 +404,7 @@ static BestPos_t xladjust(XLabels_t * xlp, object_t * objp)
     xlabel_t *lp = objp->lbl;
     double xincr = ((2 * lp->sz.x) + objp->sz.x) / XLXDENOM;
     double yincr = ((2 * lp->sz.y) + objp->sz.y) / XLYDENOM;
-    object_t *intrsx[XLNBR] = {NULL};
+    object_t *intrsx[XLNBR] = {0};
     BestPos_t bp, nbp;
 
     assert(objp->lbl);
