@@ -20,7 +20,7 @@ extern int compute_y_coords(vtx_data*, int, double*, int);
 extern int compute_hierarchy(vtx_data*, int, double, double, 
                                 double*, int**, int**, int*); 
 extern int IMDS_given_dim(vtx_data*, int, double*, double*, double);
-extern int stress_majorization_with_hierarchy(vtx_data*, int, int, double**, 
+extern int stress_majorization_with_hierarchy(vtx_data*, int, double**, 
                                               node_t**, int, int, int, int, double);
 #ifdef IPSEPCOLA
 typedef struct ipsep_options {
@@ -40,7 +40,7 @@ typedef struct ipsep_options {
 } ipsep_options;
 
  /* stress majorization, for Constraint Layout */
-extern int stress_majorization_cola(vtx_data*, int, int, double**, node_t**, int, int, int, ipsep_options*);
+extern int stress_majorization_cola(vtx_data*, int, double**, node_t**, int, int, int, ipsep_options*);
 #endif
 #endif
 
