@@ -17,17 +17,20 @@
 
 ******************************************/
 
-
+#include <assert.h>
+#include <cgraph/bitarray.h>
+#include <common/memory.h>
+#include <float.h>
 #include <neatogen/bfs.h>
 #include <neatogen/dijkstra.h>
 #include <limits.h>
+#include <stdbool.h>
 #include <stdlib.h>
-/* #include <math.h> */
-
-#define MAX_DIST (double)INT_MAX
 
 typedef DistType Word;
 
+#define MAX_DIST ((DistType)INT_MAX)
+
 /* This heap class is suited to the Dijkstra alg.
    data[i]=vertexNum <==> index[vertexNum]=i
 */
@@ -97,10 +100,10 @@ initHeap(heap * h, int startVertex, int index[], Word dist[], int n)
 	heapify(h, j, index, dist);
 }
 
-static boolean extractMax(heap * h, int *max, int index[], Word dist[])
+static bool extractMax(heap * h, int *max, int index[], Word dist[])
 {
     if (h->heapSize == 0)
-	return FALSE;
+	return false;
 
     *max = h->data[0];
     h->data[0] = h->data[h->heapSize - 1];
@@ -108,7 +111,7 @@ static boolean extractMax(heap * h, int *max, int index[], Word dist[])
     h->heapSize--;
     heapify(h, 0, index, dist);
 
-    return TRUE;
+    return true;
 }
 
 static void
@@ -139,14 +142,13 @@ void dijkstra(int vertex, vtx_data * graph, int n, DistType * dist)
     int i;
     heap H;
     int closestVertex, neighbor;
-    DistType closestDist, prevClosestDist = INT_MAX;
-    static int *index;
+    DistType closestDist, prevClosestDist = MAX_DIST;
 
-    index = realloc(index, n * sizeof(int));
+    int *index = gcalloc(n, sizeof(int));
 
     /* initial distances with edge weights: */
     for (i = 0; i < n; i++)
-	dist[i] = (DistType) MAX_DIST;
+	dist[i] = MAX_DIST;
     dist[vertex] = 0;
     for (i = 1; i < graph[vertex].nedges; i++)
 	dist[graph[vertex].edges[i]] = (DistType) graph[vertex].ewgts[i];
@@ -159,10 +161,8 @@ void dijkstra(int vertex, vtx_data * graph, int n, DistType * dist)
 	    break;
 	for (i = 1; i < graph[closestVertex].nedges; i++) {
 	    neighbor = graph[closestVertex].edges[i];
-	    increaseKey(&H, neighbor,
-			closestDist +
-			(DistType) graph[closestVertex].ewgts[i], index,
-			dist);
+	    increaseKey(&H, neighbor, closestDist +
+			(DistType)graph[closestVertex].ewgts[i], index, dist);
 	}
 	prevClosestDist = closestDist;
     }
@@ -172,6 +172,7 @@ void dijkstra(int vertex, vtx_data * graph, int n, DistType * dist)
 	if (dist[i] == MAX_DIST)	/* 'i' is not connected to 'vertex' */
 	    dist[i] = prevClosestDist + 10;
     freeHeap(&H);
+    free(index);
 }
 
  /* Dijkstra bounded to nodes in *unweighted* radius */
@@ -184,9 +185,6 @@ dijkstra_bounded(int vertex, vtx_data * graph, int n, DistType * dist,
 {
     int num_visited_nodes;
     int i;
-    static boolean *node_in_neighborhood = NULL;
-    static int size = 0;
-    static int *index;
     Queue Q;
     heap H;
     int closestVertex, neighbor;
@@ -200,24 +198,17 @@ dijkstra_bounded(int vertex, vtx_data * graph, int n, DistType * dist,
 	dist[i] = -1;		/* far, TOO COSTLY (O(n))! */
     }
     num_visited_nodes =
-	bfs_bounded(vertex, graph, n, dist, &Q, bound, visited_nodes);
-    if (size < n) {
-	node_in_neighborhood = realloc(node_in_neighborhood, n * sizeof(boolean));
-	for (i = size; i < n; i++) {
-	    node_in_neighborhood[i] = FALSE;
-	}
-	size = n;
-    }
+	bfs_bounded(vertex, graph, dist, &Q, bound, visited_nodes);
+    bitarray_t node_in_neighborhood = bitarray_new_or_exit(n);
     for (i = 0; i < num_visited_nodes; i++) {
-	node_in_neighborhood[visited_nodes[i]] = TRUE;
+	bitarray_set(&node_in_neighborhood, visited_nodes[i], true);
     }
 
-
-    index = realloc(index, n * sizeof(int));
+    int *index = gcalloc(n, sizeof(int));
 
     /* initial distances with edge weights: */
     for (i = 0; i < n; i++)	/* far, TOO COSTLY (O(n))! */
-	dist[i] = (DistType) MAX_DIST;
+	dist[i] = MAX_DIST;
     dist[vertex] = 0;
     for (i = 1; i < graph[vertex].nedges; i++)
 	dist[graph[vertex].edges[i]] = (DistType) graph[vertex].ewgts[i];
@@ -227,7 +218,7 @@ dijkstra_bounded(int vertex, vtx_data * graph, int n, DistType * dist,
 
     while (num_found < num_visited_nodes
 	   && extractMax(&H, &closestVertex, index, dist)) {
-	if (node_in_neighborhood[closestVertex]) {
+	if (bitarray_get(node_in_neighborhood, closestVertex)) {
 	    num_found++;
 	}
 	closestDist = dist[closestVertex];
@@ -235,18 +226,14 @@ dijkstra_bounded(int vertex, vtx_data * graph, int n, DistType * dist,
 	    break;
 	for (i = 1; i < graph[closestVertex].nedges; i++) {
 	    neighbor = graph[closestVertex].edges[i];
-	    increaseKey(&H, neighbor,
-			closestDist +
-			(DistType) graph[closestVertex].ewgts[i], index,
-			dist);
+	    increaseKey(&H, neighbor, closestDist +
+			(DistType)graph[closestVertex].ewgts[i], index, dist);
 	}
     }
 
-    /* restore initial false-status of 'node_in_neighborhood' */
-    for (i = 0; i < num_visited_nodes; i++) {
-	node_in_neighborhood[visited_nodes[i]] = FALSE;
-    }
+    bitarray_reset(&node_in_neighborhood);
     freeHeap(&H);
+    free(index);
     freeQueue(&Q);
     return num_visited_nodes;
 }
@@ -291,10 +278,10 @@ initHeap_f(heap * h, int startVertex, int index[], float dist[], int n)
 	heapify_f(h, j, index, dist);
 }
 
-static boolean extractMax_f(heap * h, int *max, int index[], float dist[])
+static bool extractMax_f(heap * h, int *max, int index[], float dist[])
 {
     if (h->heapSize == 0)
-	return FALSE;
+	return false;
 
     *max = h->data[0];
     h->data[0] = h->data[h->heapSize - 1];
@@ -302,7 +289,7 @@ static boolean extractMax_f(heap * h, int *max, int index[], float dist[])
     h->heapSize--;
     heapify_f(h, 0, index, dist);
 
-    return TRUE;
+    return true;
 }
 
 static void
@@ -344,7 +331,7 @@ void dijkstra_f(int vertex, vtx_data * graph, int n, float *dist)
 
     /* initial distances with edge weights: */
     for (i = 0; i < n; i++)
-	dist[i] = MAXFLOAT;
+	dist[i] = FLT_MAX;
     dist[vertex] = 0;
     for (i = 1; i < graph[vertex].nedges; i++)
 	dist[graph[vertex].edges[i]] = graph[vertex].ewgts[i];
@@ -353,12 +340,11 @@ void dijkstra_f(int vertex, vtx_data * graph, int n, float *dist)
 
     while (extractMax_f(&H, &closestVertex, index, dist)) {
 	closestDist = dist[closestVertex];
-	if (closestDist == MAXFLOAT)
+	if (closestDist == FLT_MAX)
 	    break;
 	for (i = 1; i < graph[closestVertex].nedges; i++) {
 	    neighbor = graph[closestVertex].edges[i];
-	    increaseKey_f(&H, neighbor,
-			  closestDist + graph[closestVertex].ewgts[i],
+	    increaseKey_f(&H, neighbor, closestDist + graph[closestVertex].ewgts[i],
 			  index, dist);
 	}
     }
@@ -374,36 +360,39 @@ int dijkstra_sgd(graph_sgd *graph, int source, term_sgd *terms) {
     heap h;
     int *indices = N_GNEW(graph->n, int);
     float *dists = N_GNEW(graph->n, float);
-    int i;
-    for (i=0; i<graph->n; i++) {
-        dists[i] = MAXFLOAT;
+    for (size_t i= 0; i < graph->n; i++) {
+        dists[i] = FLT_MAX;
     }
     dists[source] = 0;
-    for (i=graph->sources[source]; i<graph->sources[source+1]; i++) {
-        int target = graph->targets[i];
+    for (size_t i = graph->sources[source]; i < graph->sources[source + 1];
+         i++) {
+        size_t target = graph->targets[i];
         dists[target] = graph->weights[i];
     }
-    initHeap_f(&h, source, indices, dists, graph->n);
+    assert(graph->n <= INT_MAX);
+    initHeap_f(&h, source, indices, dists, (int)graph->n);
 
     int closest = 0, offset = 0;
     while (extractMax_f(&h, &closest, indices, dists)) {
         float d = dists[closest];
-        if (d == MAXFLOAT) {
+        if (d == FLT_MAX) {
             break;
         }
         // if the target is fixed then always create a term as shortest paths are not calculated from there
         // if not fixed then only create a term if the target index is lower
-        if (graph->pinneds[closest] || closest<source) {
+        if (bitarray_get(graph->pinneds, closest) || closest<source) {
             terms[offset].i = source;
             terms[offset].j = closest;
             terms[offset].d = d;
             terms[offset].w = 1 / (d*d);
             offset++;
         }
-        for (i=graph->sources[closest]; i<graph->sources[closest+1]; i++) {
-            int target = graph->targets[i];
+        for (size_t i = graph->sources[closest]; i < graph->sources[closest + 1];
+             i++) {
+            size_t target = graph->targets[i];
             float weight = graph->weights[i];
-            increaseKey_f(&h, target, d+weight, indices, dists);
+            assert(target <= (size_t)INT_MAX);
+            increaseKey_f(&h, (int)target, d+weight, indices, dists);
         }
     }
     freeHeap(&h);
@@ -411,4 +400,3 @@ int dijkstra_sgd(graph_sgd *graph, int source, term_sgd *terms) {
     free(dists);
     return offset;
 }
-
