@@ -17,24 +17,23 @@
 #include <pathplan/pathplan.h>
 #include <pathplan/vispath.h>
 #include <neatogen/multispline.h>
+#include <stdbool.h>
 
 #ifdef ORTHO
 #include <ortho/ortho.h>
 #endif
 
-extern int in_poly(Ppoly_t argpoly, Ppoint_t q);
 
-
-static boolean spline_merge(node_t * n)
+static bool spline_merge(node_t * n)
 {
     (void)n;
-    return FALSE;
+    return false;
 }
 
-static boolean swap_ends_p(edge_t * e)
+static bool swap_ends_p(edge_t * e)
 {
     (void)e;
-    return FALSE;
+    return false;
 }
 
 static splineInfo sinfo = {.swapEnds = swap_ends_p,
@@ -115,8 +114,8 @@ static void *newitem(Dt_t * d, edgeitem * obj, Dtdisc_t * disc)
 {
     edgeitem *newp;
 
-    NOTUSED(d);
-    NOTUSED(disc);
+    (void)d;
+    (void)disc;
     newp = NEW(edgeitem);
     newp->id = obj->id;
     newp->e = obj->e;
@@ -127,16 +126,16 @@ static void *newitem(Dt_t * d, edgeitem * obj, Dtdisc_t * disc)
 
 static void freeitem(Dt_t * d, edgeitem * obj, Dtdisc_t * disc)
 {
-    NOTUSED(d);
-    NOTUSED(disc);
+    (void)d;
+    (void)disc;
     free(obj);
 }
 
 static int
 cmpitems(Dt_t * d, edgeinfo * key1, edgeinfo * key2, Dtdisc_t * disc)
 {
-    NOTUSED(d);
-    NOTUSED(disc);
+    (void)d;
+    (void)disc;
 
     if (key1->n1 > key2->n1)
 	return 1;
@@ -234,14 +233,14 @@ static edge_t *equivEdge(Dt_t * map, edge_t * e)
  * We have to handle port labels here.
  * as well as update the bbox from edge labels.
  */
-void makeSelfArcs(path * P, edge_t * e, int stepx)
+void makeSelfArcs(edge_t * e, int stepx)
 {
     int cnt = ED_count(e);
 
     if (cnt == 1 || Concentrate) {
 	edge_t *edges1[1];
 	edges1[0] = e;
-	makeSelfEdge(P, edges1, 0, 1, stepx, stepx, &sinfo);
+	makeSelfEdge(edges1, 0, 1, stepx, stepx, &sinfo);
 	if (ED_label(e))
 	    updateBB(agraphof(agtail(e)), ED_label(e));
 	makePortLabels(e);
@@ -252,7 +251,7 @@ void makeSelfArcs(path * P, edge_t * e, int stepx)
 	    edges[i] = e;
 	    e = ED_to_virt(e);
 	}
-	makeSelfEdge(P, edges, 0, cnt, stepx, stepx, &sinfo);
+	makeSelfEdge(edges, 0, cnt, stepx, stepx, &sinfo);
 	for (i = 0; i < cnt; i++) {
 	    e = edges[i];
 	    if (ED_label(e))
@@ -275,7 +274,7 @@ void makeSelfArcs(path * P, edge_t * e, int stepx)
  * The polygon has its vertices in CW order.
  * 
  */
-Ppoly_t *makeObstacle(node_t * n, expand_t* pmargin, boolean isOrtho)
+Ppoly_t *makeObstacle(node_t * n, expand_t* pmargin, bool isOrtho)
 {
     Ppoly_t *obs;
     polygon_t *poly;
@@ -442,17 +441,13 @@ Ppoly_t *makeObstacle(node_t * n, expand_t* pmargin, boolean isOrtho)
 
 /* getPath
  * Construct the shortest path from one endpoint of e to the other.
- * The obstacles and their number are given by obs and npoly.
  * vconfig is a precomputed data structure to help in the computation.
  * If chkPts is true, the function finds the polygons, if any, containing
  * the endpoints and tells the shortest path computation to ignore them. 
  * Assumes this info is set in ND_lim, usually from _spline_edges.
  * Returns the shortest path.
  */
-Ppolyline_t
-getPath(edge_t * e, vconfig_t * vconfig, int chkPts, Ppoly_t ** obs,
-	int npoly)
-{
+Ppolyline_t getPath(edge_t *e, vconfig_t *vconfig, int chkPts) {
     Ppolyline_t line;
     int pp, qp;
     Ppoint_t p, q;
@@ -472,19 +467,14 @@ getPath(edge_t * e, vconfig_t * vconfig, int chkPts, Ppoly_t ** obs,
 
 /* makePolyline:
  */
-static void
-makePolyline(graph_t* g, edge_t * e)
-{
+static void makePolyline(edge_t * e) {
     Ppolyline_t spl, line = ED_path(e);
-    Ppoint_t p0, q0;
 
-    p0 = line.ps[0];
-    q0 = line.ps[line.pn - 1];
     make_polyline (line, &spl);
     if (Verbose > 1)
 	fprintf(stderr, "polyline %s %s\n", agnameof(agtail(e)), agnameof(aghead(e)));
     clip_and_install(e, aghead(e), spl.ps, spl.pn, &sinfo);
-    addEdgeLabels(g, e, p0, q0);
+    addEdgeLabels(e);
 }
 
 /* makeSpline:
@@ -497,8 +487,7 @@ makePolyline(graph_t* g, edge_t * e)
  * is on or inside one of the obstacles and, if so, tells the shortest path
  * computation to ignore them. 
  */
-void makeSpline(graph_t* g, edge_t * e, Ppoly_t ** obs, int npoly, boolean chkPts)
-{
+void makeSpline(edge_t *e, Ppoly_t **obs, int npoly, bool chkPts) {
     Ppolyline_t line, spline;
     Pvector_t slopes[2];
     int i, n_barriers;
@@ -532,7 +521,7 @@ void makeSpline(graph_t* g, edge_t * e, Ppoly_t ** obs, int npoly, boolean chkPt
 	fprintf(stderr, "spline %s %s\n", agnameof(agtail(e)), agnameof(aghead(e)));
     clip_and_install(e, aghead(e), spline.ps, spline.pn, &sinfo);
     free(barriers);
-    addEdgeLabels(g, e, p, q);
+    addEdgeLabels(e);
 }
 
   /* True if either head or tail has a port on its boundary */
@@ -546,7 +535,7 @@ void makeSpline(graph_t* g, edge_t * e, Ppoly_t ** obs, int npoly, boolean chkPt
  * remain in the cluster's bounding box and, conversely, a cluster's box
  * is not altered to reflect intra-cluster edges.
  * If Nop > 1 and the spline exists, it is just copied.
- * NOTE: if edgetype = ET_NONE, we shouldn't be here.
+ * NOTE: if edgetype = EDGETYPE_NONE, we shouldn't be here.
  */
 static int _spline_edges(graph_t * g, expand_t* pmargin, int edgetype)
 {
@@ -566,10 +555,10 @@ static int _spline_edges(graph_t * g, expand_t* pmargin, int edgetype)
 #endif
     
     /* build configuration */
-    if (edgetype >= ET_PLINE) {
+    if (edgetype >= EDGETYPE_PLINE) {
 	obs = N_NEW(agnnodes(g), Ppoly_t *);
 	for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
-	    obp = makeObstacle(n, pmargin, edgetype == ET_ORTHO);
+	    obp = makeObstacle(n, pmargin, edgetype == EDGETYPE_ORTHO);
 	    if (obp) {
 		ND_lim(n) = i; 
 		obs[i++] = obp;
@@ -583,10 +572,10 @@ static int _spline_edges(graph_t * g, expand_t* pmargin, int edgetype)
     npoly = i;
     if (obs) {
 	if ((legal = Plegal_arrangement(obs, npoly))) {
-	    if (edgetype != ET_ORTHO) vconfig = Pobsopen(obs, npoly);
+	    if (edgetype != EDGETYPE_ORTHO) vconfig = Pobsopen(obs, npoly);
 	}
 	else {
-	    if (edgetype == ET_ORTHO)
+	    if (edgetype == EDGETYPE_ORTHO)
 		agerr(AGWARN, "the bounding boxes of some nodes touch - falling back to straight line edges\n");
 	    else 
 		agerr(AGWARN, "some nodes with margin (%.02f,%.02f) touch - falling back to straight line edges\n", pmargin->x, pmargin->y);
@@ -596,19 +585,19 @@ static int _spline_edges(graph_t * g, expand_t* pmargin, int edgetype)
     /* route edges  */
     if (Verbose)
 	fprintf(stderr, "Creating edges using %s\n",
-	    (legal && edgetype == ET_ORTHO) ? "orthogonal lines" :
-	    (vconfig ? (edgetype == ET_SPLINE ? "splines" : "polylines") : 
+	    (legal && edgetype == EDGETYPE_ORTHO) ? "orthogonal lines" :
+	    (vconfig ? (edgetype == EDGETYPE_SPLINE ? "splines" : "polylines") :
 		"line segments"));
     if (vconfig) {
 	/* path-finding pass */
 	for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
 	    for (e = agfstout(g, n); e; e = agnxtout(g, e)) {
-		ED_path(e) = getPath(e, vconfig, TRUE, obs, npoly);
+		ED_path(e) = getPath(e, vconfig, TRUE);
 	    }
 	}
     }
 #ifdef ORTHO
-    else if (legal && edgetype == ET_ORTHO) {
+    else if (legal && edgetype == EDGETYPE_ORTHO) {
 	orthoEdges (g, 0);
 	useEdges = 1;
     }
@@ -620,9 +609,9 @@ static int _spline_edges(graph_t * g, expand_t* pmargin, int edgetype)
 /* fprintf (stderr, "%s -- %s %d\n", agnameof(agtail(e)), agnameof(aghead(e)), ED_count(e)); */
 	    node_t *head = aghead(e);
 	    if (useEdges && ED_spl(e)) {
-		addEdgeLabels(g, e,
-			      add_pointf(ND_coord(n), ED_tail_port(e).p),
-			      add_pointf(ND_coord(head), ED_head_port(e).p));
+		add_pointf(ND_coord(n), ED_tail_port(e).p);
+		add_pointf(ND_coord(head), ED_head_port(e).p);
+		addEdgeLabels(e);
 	    } 
 	    else if (ED_count(e) == 0) continue;  /* only do representative */
 	    else if (n == head) {    /* self arc */
@@ -630,8 +619,8 @@ static int _spline_edges(graph_t * g, expand_t* pmargin, int edgetype)
 		    P = NEW(path);
 		    P->boxes = N_NEW(agnnodes(g) + 20 * 2 * 9, boxf);
 		}
-		makeSelfArcs(P, e, GD_nodesep(g->root));
-	    } else if (vconfig) { /* ET_SPLINE or ET_PLINE */
+		makeSelfArcs(e, GD_nodesep(g->root));
+	    } else if (vconfig) { /* EDGETYPE_SPLINE or EDGETYPE_PLINE */
 #ifdef HAVE_GTS
 		if (ED_count(e) > 1 || BOUNDARY_PORT(e)) {
 		    int fail = 0;
@@ -640,7 +629,7 @@ static int _spline_edges(graph_t * g, expand_t* pmargin, int edgetype)
 			makeStraightEdge(g, e, edgetype, &sinfo);
 		    else { 
 			if (!rtr) rtr = mkRouter (obs, npoly);
-			fail = makeMultiSpline(g, e, rtr, edgetype == ET_PLINE);
+			fail = makeMultiSpline(e, rtr, edgetype == EDGETYPE_PLINE);
 		    } 
 		    if (!fail) continue;
 		}
@@ -653,10 +642,10 @@ static int _spline_edges(graph_t * g, expand_t* pmargin, int edgetype)
 		if (Concentrate) cnt = 1; /* only do representative */
 		e0 = e;
 		for (i = 0; i < cnt; i++) {
-		    if (edgetype == ET_SPLINE)
-			makeSpline(g, e0, obs, npoly, TRUE);
+		    if (edgetype == EDGETYPE_SPLINE)
+			makeSpline(e0, obs, npoly, true);
 		    else
-			makePolyline(g, e0);
+			makePolyline(e0);
 		    e0 = ED_to_virt(e0);
 		}
 	    } else {
@@ -767,17 +756,16 @@ int spline_edges1(graph_t * g, int edgetype)
  * when output in dot or plain format.
  *
  */
-void spline_edges0(graph_t * g, boolean set_aspect)
-{
+void spline_edges0(graph_t *g, bool set_aspect) {
     int et = EDGE_TYPE (g);
     if (set_aspect) neato_set_aspect(g);
-    if (et == ET_NONE) return;
+    if (et == EDGETYPE_NONE) return;
 #ifndef ORTHO
-    if (et == ET_ORTHO) {
+    if (et == EDGETYPE_ORTHO) {
 	agerr (AGWARN, "Orthogonal edges not yet supported\n");
-	et = ET_PLINE; 
-	GD_flags(g->root) &= ~ET_ORTHO;
-	GD_flags(g->root) |= ET_PLINE;
+	et = EDGETYPE_PLINE;
+	GD_flags(g->root) &= ~EDGETYPE_ORTHO;
+	GD_flags(g->root) |= EDGETYPE_PLINE;
     }
 #endif
     spline_edges1(g, et);
@@ -818,7 +806,7 @@ void spline_edges(graph_t * g)
     }
 	
     shiftClusters (g, GD_bb(g).LL);
-    spline_edges0(g, TRUE);
+    spline_edges0(g, true);
 }
 
 /* scaleEdge:
@@ -1002,18 +990,18 @@ void neato_translate(Agraph_t * g)
  * Return false if no transform is performed. This includes
  * the possibility that a translation was done.
  */
-static boolean _neato_set_aspect(graph_t * g)
+static bool _neato_set_aspect(graph_t * g)
 {
     double xf, yf, actual, desired;
     node_t *n;
-    boolean translated = FALSE;
+    bool translated = false;
 
     if (g->root != g)
-	return FALSE;
+	return false;
 
     if (GD_drawing(g)->ratio_kind) {
 	if (GD_bb(g).LL.x || GD_bb(g).LL.y) {
-	    translated = TRUE;
+	    translated = true;
 	    neato_translate (g);
 	}
 	/* normalize */
@@ -1082,10 +1070,10 @@ static boolean _neato_set_aspect(graph_t * g)
 	    ND_pos(n)[1] *= yf;
 	}
 	scaleBB(g, xf, yf);
-        return TRUE;
+        return true;
     }
     else
-	return FALSE;
+	return false;
 }
 
 /* neato_set_aspect:
@@ -1095,10 +1083,10 @@ static boolean _neato_set_aspect(graph_t * g)
  *
  * Return true if some node moved.
  */
-boolean neato_set_aspect(graph_t * g)
+bool neato_set_aspect(graph_t * g)
 {
     node_t *n;
-    boolean moved = FALSE;
+    bool moved = false;
 
 	/* setting aspect ratio only makes sense on root graph */
     moved = _neato_set_aspect(g);
