@@ -9,7 +9,9 @@
  *************************************************************************/
 
 #include "config.h"
+#include "gdioctx_wrapper.h"
 
+#include <stdbool.h>
 #include <stdlib.h>
 #include <stdint.h>
 #include <string.h>
@@ -20,8 +22,6 @@
 #include <gvc/gvcint.h>	/* for gvc->g for agget */
 #include <gd.h>
 
-#define NOTUSED(x)	(void) (x)
-
 typedef enum {
 	FORMAT_GIF,
 	FORMAT_JPEG,
@@ -32,14 +32,14 @@ typedef enum {
 	FORMAT_XBM,
 } format_type;
 
-extern boolean mapbool(char *);
+extern bool mapbool(const char *);
 extern pointf Bezier(pointf * V, int degree, double t, pointf * Left, pointf * Right);
 
 #define BEZIERSUBDIVISION 10
 
 static void gdgen_resolve_color(GVJ_t * job, gvcolor_t * color)
 {
-    gdImagePtr im = (gdImagePtr) job->context;
+    gdImagePtr im = job->context;
     int alpha;
 
     if (!im)
@@ -59,14 +59,14 @@ static void gdgen_resolve_color(GVJ_t * job, gvcolor_t * color)
     color->type = COLOR_INDEX;
 }
 
-static int white, black, transparent, basecolor;
+static int transparent, basecolor;
 
 #define GD_XYMAX INT32_MAX
 
 static void gdgen_begin_page(GVJ_t * job)
 {
     char *bgcolor_str = NULL, *truecolor_str = NULL;
-    boolean truecolor_p = FALSE;	/* try to use cheaper paletted mode */
+    bool truecolor_p = false;	/* try to use cheaper paletted mode */
     gdImagePtr im = NULL;
 
     truecolor_str = agget(job->gvc->g, "truecolor");	/* allow user to force truecolor */
@@ -77,16 +77,16 @@ static void gdgen_begin_page(GVJ_t * job)
 
     if (bgcolor_str && strcmp(bgcolor_str, "transparent") == 0) {
 	if (job->render.features->flags & GVDEVICE_DOES_TRUECOLOR)
-	    truecolor_p = TRUE;	/* force truecolor */
+	    truecolor_p = true;	/* force truecolor */
     }
 
     if (GD_has_images(job->gvc->g))
-	truecolor_p = TRUE;	/* force truecolor */
+	truecolor_p = true;	/* force truecolor */
 
     if (job->external_context) {
 	if (job->common->verbose)
 	    fprintf(stderr, "%s: using existing GD image\n", job->common->cmdname);
-	im = (gdImagePtr) (job->context);
+	im = job->context;
     } else {
         if (job->width * job->height >= GD_XYMAX) {
 	    double scale = sqrt(GD_XYMAX / (job->width * job->height));
@@ -114,7 +114,7 @@ static void gdgen_begin_page(GVJ_t * job)
 			job->width, job->height);
 	    im = gdImageCreate(job->width, job->height);
 	}
-        job->context = (void *) im;
+        job->context = im;
     }
 
     if (!im) {
@@ -129,12 +129,6 @@ static void gdgen_begin_page(GVJ_t * job)
 					   gdBlueMax, gdAlphaTransparent);
     gdImageColorTransparent(im, transparent);
 
-    white = gdImageColorResolveAlpha(im,
-				     gdRedMax, gdGreenMax, gdBlueMax,
-				     gdAlphaOpaque);
-
-    black = gdImageColorResolveAlpha(im, 0, 0, 0, gdAlphaOpaque);
-
     /* Blending must be off to lay a transparent basecolor.
        Nothing to blend with anyway. */
     gdImageAlphaBlending(im, FALSE);
@@ -149,13 +143,13 @@ extern void gvdevice_gd_putC (gdIOCtx *context, int C);
 
 static void gdgen_end_page(GVJ_t * job)
 {
-    gdImagePtr im = (gdImagePtr) job->context;
+    gdImagePtr im = job->context;
 
-    gdIOCtx ctx;
+    gd_context_t gd_context = {{0}};
 
-    ctx.putBuf = gvdevice_gd_putBuf;
-    ctx.putC = gvdevice_gd_putC;
-    ctx.tell = (void*)job;    /* hide *job here */
+    gd_context.ctx.putBuf = gvdevice_gd_putBuf;
+    gd_context.ctx.putC = gvdevice_gd_putC;
+    gd_context.job = job;
 
     if (!im)
 	return;
@@ -168,14 +162,14 @@ static void gdgen_end_page(GVJ_t * job)
 	/* Only save the alpha channel in outputs that support it if
 	   the base color was transparent.   Otherwise everything
 	   was blended so there is no useful alpha info */
-	gdImageSaveAlpha(im, (basecolor == transparent));
+	gdImageSaveAlpha(im, basecolor == transparent);
 	switch (job->render.id) {
 	case FORMAT_GIF:
 #ifdef HAVE_GD_GIF
 	    gdImageTrueColorToPalette(im, 0, 256);
-	    gdImageGifCtx(im, &ctx);
+	    gdImageGifCtx(im, &gd_context.ctx);
 #else
-            NOTUSED(ctx);
+            (void)gd_context;
 #endif
 	    break;
 	case FORMAT_JPEG:
@@ -189,13 +183,13 @@ static void gdgen_end_page(GVJ_t * job)
 	     * be near optimal for many applications).  See the IJG JPEG
 	     * library documentation for more details.  */
 #define JPEG_QUALITY -1
-	    gdImageJpegCtx(im, &ctx, JPEG_QUALITY);
+	    gdImageJpegCtx(im, &gd_context.ctx, JPEG_QUALITY);
 #endif
 
 	    break;
 	case FORMAT_PNG:
 #ifdef HAVE_GD_PNG
-	    gdImagePngCtx(im, &ctx);
+	    gdImagePngCtx(im, &gd_context.ctx);
 #endif
 	    break;
 
@@ -204,7 +198,7 @@ static void gdgen_end_page(GVJ_t * job)
 	    {
 	        /* Use black for the foreground color for the B&W wbmp image. */
 		int black = gdImageColorResolveAlpha(im, 0, 0, 0, gdAlphaOpaque);
-		gdImageWBMPCtx(im, black, &ctx);
+		gdImageWBMPCtx(im, black, &gd_context.ctx);
 	    }
 	    break;
 #endif
@@ -216,19 +210,12 @@ static void gdgen_end_page(GVJ_t * job)
 #ifdef HAVE_LIBZ
 	case FORMAT_GD2:
 #define GD2_CHUNKSIZE 128
-#define GD2_RAW 1
 #define GD2_COMPRESSED 2
 	    gdImageGd2(im, job->output_file, GD2_CHUNKSIZE, GD2_COMPRESSED);
 	    break;
 #endif
 
 	case FORMAT_XBM:
-#if 0
-/* libgd support only reading .xpm files */
-#ifdef HAVE_GD_XPM
-	    gdImageXbm(im, job->output_file);
-#endif
-#endif
 	    break;
 	}
 	gdImageDestroy(im);
@@ -247,29 +234,15 @@ static void gdgen_missingfont(char *err, char *fontreq)
 
     if (n_errors >= 20)
 	return;
-    if ((lastmissing == 0) || (strcmp(lastmissing, fontreq))) {
-#ifdef HAVE_GD_FONTCONFIG
-#if 0
-/* FIXME - error function */
-	agerr(AGERR, "%s : %s\n", err, fontreq);
-#endif
-#else
+    if (lastmissing == 0 || strcmp(lastmissing, fontreq)) {
+#ifndef HAVE_GD_FONTCONFIG
 	char *p = getenv("GDFONTPATH");
 	if (!p)
 	    p = DEFAULT_FONTPATH;
-#if 0
-/* FIXME - error function */
-	agerr(AGERR, "%s : %s in %s\n", err, fontreq, p);
-#endif
 #endif
 	free(lastmissing);
 	lastmissing = strdup(fontreq);
 	n_errors++;
-#if 0
-/* FIXME - error function */
-	if (n_errors >= 20)
-	    agerr(AGWARN, "(font errors suppressed)\n");
-#endif
     }
 }
 #endif
@@ -279,12 +252,12 @@ static void gdgen_missingfont(char *err, char *fontreq)
 /* fontsize at which text is rendered by a simple line */
 #define FONTSIZE_TOO_SMALL 1.5
 
-#ifdef _WIN32
-#   define GD_IMPORT __declspec(dllimport) extern
+#ifdef GVDLL
+#define GD_IMPORT __declspec(dllimport)
 #else
-#   define GD_IMPORT extern
+#define GD_IMPORT
 #endif
-GD_IMPORT gdFontPtr gdFontTiny, gdFontSmall, gdFontMediumBold, gdFontLarge, gdFontGiant;
+GD_IMPORT extern gdFontPtr gdFontTiny, gdFontSmall, gdFontMediumBold, gdFontLarge, gdFontGiant;
 
 void gdgen_text(gdImagePtr im, pointf spf, pointf epf, int fontcolor, double fontsize, int fontdpi, double fontangle, char *fontname, char *str)
 {
@@ -346,7 +319,7 @@ extern char* gd_psfontResolve (PostscriptAlias* pa);
 
 static void gdgen_textspan(GVJ_t * job, pointf p, textspan_t * span)
 {
-    gdImagePtr im = (gdImagePtr) job->context;
+    gdImagePtr im = job->context;
     pointf spf, epf;
     double spanwidth = span->size.x * job->zoom * job->dpi.x / POINTS_PER_INCH;
     char* fontname;
@@ -452,10 +425,10 @@ gdgen_bezier(GVJ_t * job, pointf * A, int n, int arrow_at_start,
 	     int arrow_at_end, int filled)
 {
     obj_state_t *obj = job->obj;
-    gdImagePtr im = (gdImagePtr) job->context;
+    gdImagePtr im = job->context;
     pointf p0, p1, V[4];
     int i, j, step, pen;
-    boolean pen_ok, fill_ok;
+    bool pen_ok, fill_ok;
     gdImagePtr brush = NULL;
     gdPoint F[4];
 
@@ -463,8 +436,8 @@ gdgen_bezier(GVJ_t * job, pointf * A, int n, int arrow_at_start,
 	return;
 
     pen = gdgen_set_penstyle(job, im, &brush);
-    pen_ok = (pen != gdImageGetTransparent(im));
-    fill_ok = (filled && obj->fillcolor.u.index != gdImageGetTransparent(im));
+    pen_ok = pen != gdImageGetTransparent(im);
+    fill_ok = filled && obj->fillcolor.u.index != gdImageGetTransparent(im);
 
     if (pen_ok || fill_ok) {
         V[3] = A[0];
@@ -497,18 +470,18 @@ static int points_allocated;
 static void gdgen_polygon(GVJ_t * job, pointf * A, int n, int filled)
 {
     obj_state_t *obj = job->obj;
-    gdImagePtr im = (gdImagePtr) job->context;
+    gdImagePtr im = job->context;
     gdImagePtr brush = NULL;
     int i;
     int pen;
-    boolean pen_ok, fill_ok;
+    bool pen_ok, fill_ok;
 
     if (!im)
 	return;
 
     pen = gdgen_set_penstyle(job, im, &brush);
-    pen_ok = (pen != gdImageGetTransparent(im));
-    fill_ok = (filled && obj->fillcolor.u.index != gdImageGetTransparent(im));
+    pen_ok = pen != gdImageGetTransparent(im);
+    fill_ok = filled && obj->fillcolor.u.index != gdImageGetTransparent(im);
 
     if (pen_ok || fill_ok) {
         if (n > points_allocated) {
@@ -532,18 +505,18 @@ static void gdgen_polygon(GVJ_t * job, pointf * A, int n, int filled)
 static void gdgen_ellipse(GVJ_t * job, pointf * A, int filled)
 {
     obj_state_t *obj = job->obj;
-    gdImagePtr im = (gdImagePtr) job->context;
+    gdImagePtr im = job->context;
     double dx, dy;
     int pen;
-    boolean pen_ok, fill_ok;
+    bool pen_ok, fill_ok;
     gdImagePtr brush = NULL;
 
     if (!im)
 	return;
 
     pen = gdgen_set_penstyle(job, im, &brush);
-    pen_ok = (pen != gdImageGetTransparent(im));
-    fill_ok = (filled && obj->fillcolor.u.index != gdImageGetTransparent(im));
+    pen_ok = pen != gdImageGetTransparent(im);
+    fill_ok = filled && obj->fillcolor.u.index != gdImageGetTransparent(im);
 
     dx = 2 * (A[1].x - A[0].x);
     dy = 2 * (A[1].y - A[0].y);
@@ -561,18 +534,18 @@ static void gdgen_ellipse(GVJ_t * job, pointf * A, int filled)
 
 static void gdgen_polyline(GVJ_t * job, pointf * A, int n)
 {
-    gdImagePtr im = (gdImagePtr) job->context;
+    gdImagePtr im = job->context;
     pointf p, p1;
     int i;
     int pen;
-    boolean pen_ok;
+    bool pen_ok;
     gdImagePtr brush = NULL;
 
     if (!im)
 	return;
 
     pen = gdgen_set_penstyle(job, im, &brush);
-    pen_ok = (pen != gdImageGetTransparent(im));
+    pen_ok = pen != gdImageGetTransparent(im);
 
     if (pen_ok) {
         p = A[0];
@@ -681,11 +654,5 @@ gvplugin_installed_t gvdevice_gd_types2[] = {
     {FORMAT_GD2, "gd2:gd", 1, NULL, &device_features_gd_tc_no_writer},
 #endif
 
-#if 0
-/* libgd has no support for xbm as output */
-#ifdef HAVE_GD_XPM
-    {FORMAT_XBM, "xbm:gd", 1, NULL, &device_features_gd},
-#endif
-#endif
     {0, NULL, 0, NULL, NULL}
 };
