@@ -1,6 +1,3 @@
-/* $Id$ $Revision$ */
-/* vim:set shiftwidth=4 ts=8: */
-
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
@@ -8,31 +5,34 @@
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
  *
- * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
-#include "config.h"
-
 #include <iostream>
 #include <fstream>
 #include <stdexcept>
 #include <LASi.h>
 
-#include "gvplugin_render.h"
-#include "gvplugin_device.h"
-#include "gvio.h"
-#include "gvcint.h"
-#include "agxbuf.h"
-#include "const.h"
-#include "utils.h"
-#include "ps.h"
+// Confusingly, Freetype (included transitively by LASi), ship their Autotools
+// config.h. As a result, we get macros like `HAVE_FCNTL_H` injected from their
+// headers, overriding ours. Suppress this and force ours into scope.
+#ifdef HAVE_FCNTL_H
+#undef HAVE_FCNTL_H
+#endif
+#include "config.h"
+
+#include <gvc/gvplugin_render.h>
+#include <gvc/gvplugin_device.h>
+#include <gvc/gvio.h>
+#include <gvc/gvcint.h>
+#include <cgraph/agxbuf.h>
+#include <common/const.h>
+#include <common/utils.h>
+#include "../core/ps.h"
 
 using namespace LASi;
 using namespace std;
 
-/* for CHAR_LATIN1  */
-// #include "const.h"
-
 /*
  *     J$: added `pdfmark' URL embedding.  PostScript rendered from
  *         dot files with URL attributes will get active PDF links
@@ -42,37 +42,41 @@ using namespace std;
 
 typedef enum { FORMAT_PS, FORMAT_PS2, FORMAT_EPS } format_type;
 
-//static int isLatin1;
-//static char setupLatin1;
+struct Context {
+  using write_fn = size_t(*)(GVJ_t*, const char*, size_t);
 
-PostscriptDocument *doc;
-size_t (*save_write_fn) (GVJ_t *job, const char *s, size_t len);
+  Context(write_fn fn): save_write_fn(fn) { }
+
+  PostscriptDocument doc;
+  write_fn save_write_fn;
+};
 
 static size_t lasi_head_writer(GVJ_t * job, const char *s, size_t len)
 {
-    doc->osHeader() << s;
+    Context *ctxt = reinterpret_cast<Context*>(job->context);
+    ctxt->doc.osHeader() << s;
     return len;
 }
 
 static size_t lasi_body_writer(GVJ_t * job, const char *s, size_t len)
 {
-    doc->osBody() << s;
+    Context *ctxt = reinterpret_cast<Context*>(job->context);
+    ctxt->doc.osBody() << s;
     return len;
 }
 
 static size_t lasi_footer_writer(GVJ_t * job, const char *s, size_t len)
 {
-    doc->osFooter() << s;
+    Context *ctxt = reinterpret_cast<Context*>(job->context);
+    ctxt->doc.osFooter() << s;
     return len;
 }
 
 static void lasi_begin_job(GVJ_t * job)
 {
-    doc = new PostscriptDocument;
-    save_write_fn = job->gvc->write_fn;
+    job->context = new Context(job->gvc->write_fn);
     job->gvc->write_fn = lasi_head_writer;
 
-//    gvputs(job, "%!PS-Adobe-3.0 EPSF-3.0\n");
     gvprintf(job, "%%%%Creator: %s version %s (%s)\n",
 	    job->common->info[0], job->common->info[1], job->common->info[2]);
 }
@@ -81,7 +85,6 @@ static void lasi_end_job(GVJ_t * job)
 {
     job->gvc->write_fn = lasi_footer_writer;
 
-//    gvputs(job, "%%Trailer\n");
     if (job->render.id != FORMAT_EPS)
 	gvprintf(job, "%%%%Pages: %d\n", job->common->viewNum);
     if (job->common->show_boxes == NULL)
@@ -90,7 +93,6 @@ static void lasi_end_job(GVJ_t * job)
 	        job->boundingBox.LL.x, job->boundingBox.LL.y,
 	        job->boundingBox.UR.x, job->boundingBox.UR.y);
     gvputs(job, "end\nrestore\n");
-//    gvputs(job, "%%EOF\n");
 
     {
         // create the new stream to "redirect" cout's output to
@@ -115,12 +117,13 @@ static void lasi_end_job(GVJ_t * job)
 	    std::ostream & str_;
         } swapper(cout, output);
     
-        doc->write(cout);
+        Context *ctxt = reinterpret_cast<Context*>(job->context);
+        ctxt->doc.write(cout);
     
-        job->gvc->write_fn = save_write_fn;
+        job->gvc->write_fn = ctxt->save_write_fn;
         gvputs(job, output.str().c_str());
 
-	delete doc;
+	delete ctxt;
     }
 }
 
@@ -130,8 +133,6 @@ static void lasi_begin_graph(GVJ_t * job)
 
     job->gvc->write_fn = lasi_body_writer;
 
-//    setupLatin1 = FALSE;
-
     if (job->common->viewNum == 0) {
 	gvprintf(job, "%%%%Title: %s\n", agnameof(obj->u.g));
     	if (job->render.id != FORMAT_EPS)
@@ -158,24 +159,13 @@ static void lasi_begin_graph(GVJ_t * job)
             cat_libfile(job, NULL, args);
         }
     }
-//    isLatin1 = (GD_charset(obj->u.g) == CHAR_LATIN1);
-    /* We always setup Latin1. The charset info is always output,
-     * and installing it is cheap. With it installed, we can then
-     * rely on ps_string to convert UTF-8 characters whose encoding
-     * is in the range of Latin-1 into the Latin-1 equivalent and
-     * get the expected PostScript output.
-     */
-//    if (!setupLatin1) {
-//	gvputs(job, "setupLatin1\n");	/* as defined in ps header */
-//	setupLatin1 = TRUE;
-//    }
     /*  Set base URL for relative links (for Distiller >= 3.0)  */
     if (obj->url)
 	gvprintf(job, "[ {Catalog} << /URI << /Base %s >> >>\n"
 		"/PUT pdfmark\n", ps_string(obj->url, CHAR_UTF8));
 }
 
-static void lasi_begin_layer(GVJ_t * job, char *layername, int layerNum, int numLayers)
+static void lasi_begin_layer(GVJ_t * job, char*, int layerNum, int numLayers)
 {
     gvprintf(job, "%d %d setlayer\n", layerNum, numLayers);
 }
@@ -263,7 +253,7 @@ static void lasi_end_edge(GVJ_t * job)
     gvputs(job, "grestore\n");
 }
 
-static void lasi_begin_anchor(GVJ_t *job, char *url, char *tooltip, char *target, char *id)
+static void lasi_begin_anchor(GVJ_t *job, char *url, char*, char*, char*)
 {
     obj_state_t *obj = job->obj;
 
@@ -332,7 +322,6 @@ static void ps_set_color(GVJ_t *job, gvcolor_t *color)
 
 static void lasi_textspan(GVJ_t * job, pointf p, textspan_t * span)
 {
-    char *str;
     const char *font;
     const PangoFontDescription *pango_font;
     FontStretch stretch; 
@@ -366,15 +355,37 @@ static void lasi_textspan(GVJ_t * job, pointf p, textspan_t * span)
 	switch (pango_font_description_get_variant(pango_font)) {
 	    case PANGO_VARIANT_NORMAL: variant = NORMAL_VARIANT; break;
 	    case PANGO_VARIANT_SMALL_CAPS: variant = SMALLCAPS; break;
+#if PANGO_VERSION_CHECK(1, 50, 0)
+	    case PANGO_VARIANT_ALL_SMALL_CAPS: variant = SMALLCAPS; break;
+	    case PANGO_VARIANT_PETITE_CAPS: variant = SMALLCAPS; break;
+	    case PANGO_VARIANT_ALL_PETITE_CAPS: variant = SMALLCAPS; break;
+	    case PANGO_VARIANT_UNICASE: variant = SMALLCAPS; break;
+	    case PANGO_VARIANT_TITLE_CAPS: variant = SMALLCAPS; break;
+#endif
 	}
 	switch (pango_font_description_get_weight(pango_font)) {
+#if PANGO_VERSION_CHECK(1, 24, 0)
+	    case PANGO_WEIGHT_THIN: weight = ULTRALIGHT; break; // no exact match in LASi
+#endif
 	    case PANGO_WEIGHT_ULTRALIGHT: weight = ULTRALIGHT; break;
 	    case PANGO_WEIGHT_LIGHT: weight = LIGHT; break;
+#if PANGO_VERSION_CHECK(1, 36, 7)
+	    case PANGO_WEIGHT_SEMILIGHT: weight = LIGHT; break; // no exact match in LASi
+#endif
+#if PANGO_VERSION_CHECK(1, 24, 0)
+	    case PANGO_WEIGHT_BOOK: weight = NORMAL_WEIGHT; break; // no exact match in LASi
+#endif
 	    case PANGO_WEIGHT_NORMAL: weight = NORMAL_WEIGHT; break;
+#if PANGO_VERSION_CHECK(1, 24, 0)
+	    case PANGO_WEIGHT_MEDIUM: weight = NORMAL_WEIGHT; break; // no exact match in LASi
+#endif
 	    case PANGO_WEIGHT_SEMIBOLD: weight = BOLD; break; /* no exact match in lasi */
 	    case PANGO_WEIGHT_BOLD: weight = BOLD; break;
 	    case PANGO_WEIGHT_ULTRABOLD: weight = ULTRABOLD; break;
 	    case PANGO_WEIGHT_HEAVY: weight = HEAVY; break;
+#if PANGO_VERSION_CHECK(1, 24, 0)
+	    case PANGO_WEIGHT_ULTRAHEAVY: weight = HEAVY; break; // no exact match in LASi
+#endif
 	}
     }
     else {
@@ -395,9 +406,8 @@ static void lasi_textspan(GVJ_t * job, pointf p, textspan_t * span)
     }
 
     ps_set_color(job, &(job->obj->pencolor));
-//    gvprintdouble(job, span->font->size);
-//    gvprintf(job, " /%s set_font\n", span->font->name);
-    doc->osBody() << setFont(font, style, weight, variant, stretch) << setFontSize(span->font->size) << endl;
+    Context *ctxt = reinterpret_cast<Context*>(job->context);
+    ctxt->doc.osBody() << setFont(font, style, weight, variant, stretch) << setFontSize(span->font->size) << "\n";
     switch (span->just) {
     case 'r':
         p.x -= span->size.x;
@@ -413,10 +423,7 @@ static void lasi_textspan(GVJ_t * job, pointf p, textspan_t * span)
     p.y += span->yoffset_centerline;
     gvprintpointf(job, p);
     gvputs(job, " moveto ");
-//    gvprintdouble(job, span->size.x);
-//    str = ps_string(span->str,isLatin1);
-//    gvprintf(job, " %s alignedtext\n", str);
-    doc->osBody() << show(span->str) << endl;
+    ctxt->doc.osBody() << show(span->str) << "\n";
 
 }
 
@@ -442,9 +449,7 @@ static void lasi_ellipse(GVJ_t * job, pointf * A, int filled)
     }
 }
 
-static void
-lasi_bezier(GVJ_t * job, pointf * A, int n, int arrow_at_start,
-	     int arrow_at_end, int filled)
+static void lasi_bezier(GVJ_t * job, pointf * A, int n, int, int, int filled)
 {
     int j;
 
